from annogesiclib.gff3 import Gff3Parser


def create_dict(nums, strain, utr_detect):
    nums[strain] = {}
    if utr_detect:
        types = ["all", "5'UTR_derived", "3'UTR_derived",
                 "interCDS", "intergenic", "antisense"]
    else:
        types = ["all", "intergenic", "antisense"]
    for type_ in types:
        nums[strain][type_] = {}
        for feature in ["TSS", "sRNA", "all", "RBS", "TSS_RBS",
                        "TSS_sRNA_RBS", "TSS_sRNA", "RBS_sRNA"]:
            nums[strain][type_][feature] = 0
    return nums


def plus_data(nums, strain, sorf_types, features, utr_detect):
    for sorf_type in sorf_types:
        if ((not utr_detect) and (
                (sorf_type == "intergenic") or (
                sorf_type == "antisense") or (
                sorf_type == "all"))) or (
                utr_detect):
            for feature in features:
                nums[strain][sorf_type][feature] += 1


def print_num(out, num, nums, strain, type_):
    out.write("(for strain {0}; ".format(
              float(num) / float(nums[strain]["all"]["all"])))
    if nums[strain][type_]["all"] == 0:
        out.write("for {0} - {1})\n".format(
                  type_, 0))
    else:
        out.write("for {0} - {1})\n".format(
                  type_, float(num) / float(nums[strain][type_]["all"])))


def print_stat(nums, nums_best, strain, out, utr_detect):
    out.write(strain + ":\n")
    if utr_detect:
        out.write("\ttotal sORF in this strain are {0}\n".format(
                  nums[strain]["all"]["all"]))
    for type_, features in nums[strain].items():
        out.write("\ttotal sORF of {0} sORF candidates are {1}".format(
                  type_, nums[strain][type_]["all"]))
        out.write("(for this strain - {0})\n".format(
                  float(nums[strain][type_]["all"]) /
                  float(nums[strain]["all"]["all"])))
        for feature, num in features.items():
            if feature == "TSS":
                out.write("\t\ttotal sORF which start "
                          "from TSS are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "sRNA":
                out.write("\t\ttotal sORF without overlap with "
                          "sRNA candidates are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "RBS":
                out.write("\t\ttotal sORF which related with "
                          "ribosomal binding site are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "TSS_RBS":
                out.write("\t\ttotal sORF which start from TSS and related "
                          "with ribosomal binding site are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "TSS_sRNA":
                out.write("\t\ttotal sORF which start from TSS and without "
                          "overlap with sRNA candidates are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "RBS_sRNA":
                out.write("\t\ttotal sORF which related with "
                          "ribosomal binding site and ")
                out.write("without overlap with "
                          "sRNA candidates are {0}".format(num))
                print_num(out, num, nums, strain, type_)
            elif feature == "TSS_RBS_sRNA":
                out.write("\t\ttotal sORF which start from TSS and "
                          "related with ribosomal binding site and ")
                out.write("without overlap with "
                          "sRNA candidates are {0}".format(num))
                print_num(out, num, nums, strain, type_)
        out.write("\t\tThe best sORF are {0}\n".format(
            nums_best[strain][type_]["all"]))
        out.write("\t\tThe best sORF which without overlap with "
                  "sRNA are {0}".format(nums_best[strain][type_]["sRNA"]))
        print_num(out, nums_best[strain][type_]["sRNA"],
                  nums_best, strain, type_)


def read_file(sorf_gff):
    sorfs = []
    fh = open(sorf_gff)
    for entry in Gff3Parser().entries(fh):
        sorfs.append(entry)
    sorfs = sorted(sorfs, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    fh.close()
    return sorfs


def get_stat_num(sorfs_all, utr_detect):
    strain = ""
    nums = {}
    create_dict(nums, "total", utr_detect)
    for sorf in sorfs_all:
        if strain != sorf.seq_id:
            create_dict(nums, sorf.seq_id, utr_detect)
            strain = sorf.seq_id
        if sorf.attributes["sORF_type"] == "intergenic":
            sorf_type = "intergenic"
        elif sorf.attributes["sORF_type"] == "antisense":
            sorf_type = "antisense"
        else:
            if "5utr" in sorf.attributes["sORF_type"]:
                sorf_type = "5'UTR_derived"
            elif "3utr" in sorf.attributes["sORF_type"]:
                sorf_type = "3'UTR_derived"
            elif "interCDS" in sorf.attributes["sORF_type"]:
                sorf_type = "interCDS"
        check_class(sorf, nums, sorf_type, utr_detect, strain)
    return nums


def check_class(sorf, nums, sorf_type, utr_detect, strain):
    if (sorf.attributes["with_TSS"] != "NA") and \
       (sorf.attributes["sRNA"] == "NA") and \
       (sorf.attributes["rbs"] != "NA"):
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "TSS", "sRNA", "RBS", "TSS_RBS",
                   "TSS_sRNA_RBS", "TSS_sRNA", "RBS_sRNA"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "TSS", "sRNA", "RBS", "TSS_RBS",
                   "TSS_sRNA_RBS", "TSS_sRNA", "RBS_sRNA"], utr_detect)
    elif (sorf.attributes["with_TSS"] != "NA") and \
         (sorf.attributes["sRNA"] == "NA"):
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "TSS", "sRNA", "TSS_sRNA"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "TSS", "sRNA", "TSS_sRNA"], utr_detect)
    elif (sorf.attributes["with_TSS"] != "NA") and \
         (sorf.attributes["rbs"] != "NA"):
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "TSS", "RBS", "TSS_RBS"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "TSS", "RBS", "TSS_RBS"], utr_detect)
    elif (sorf.attributes["rbs"] != "NA") and \
         (sorf.attributes["sRNA"] == "NA"):
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "RBS", "sRNA", "RBS_sRNA"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "RBS", "sRNA", "RBS_sRNA"], utr_detect)
    elif sorf.attributes["with_TSS"] != "NA":
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "TSS"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "TSS"], utr_detect)
    elif sorf.attributes["sRNA"] == "NA":
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "sRNA"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "sRNA"], utr_detect)
    elif sorf.attributes["rbs"] != "NA":
        plus_data(nums, "total", [sorf_type, "all"],
                  ["all", "RBS"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"],
                  ["all", "RBS"], utr_detect)
    else:
        plus_data(nums, "total", [sorf_type, "all"], ["all"], utr_detect)
        plus_data(nums, strain, [sorf_type, "all"], ["all"], utr_detect)


def stat(sorf_all, sorf_best, stat_file, utr_detect):
    sorfs_all = read_file(sorf_all)
    sorfs_best = read_file(sorf_best)
    nums = get_stat_num(sorfs_all, utr_detect)
    nums_best = get_stat_num(sorfs_best, utr_detect)
    out = open(stat_file, "w")
    out.write("The filtering condition for the best sORF: \n")
    out.write("1. If TSS file exists, it will select the "
              "sORF which start from TSS.\n")
    out.write("2. If TSS file exists, it will select the "
              "sORF which have a ribosomal binding site ")
    out.write("and the ribosomal binding site shoule after a TSS.\n")
    out.write("3. If sRNA file exists and you want to "
              "exclude sORF which overlap with sRNA, ")
    out.write("it will select sORF which have non-overlap with sRNA.\n\n")
    if len(nums) <= 2:
        for strain in nums.keys():
            if strain != "total":
                print_stat(nums, nums_best, strain, out, utr_detect)
    else:
        for strain in nums.keys():
            print_stat(nums, nums_best, strain, out, utr_detect)
    out.close()



import os
import sys
import time
import shutil
import copy
from glob import glob
from subprocess import call, Popen
from annogesiclib.multiparser import Multiparser
from annogesiclib.helper import Helper
from annogesiclib.converter import Converter
from annogesiclib.circRNA import detect_circrna


class CircRNADetection(object):
    '''Detection of circRNA'''

    def __init__(self, args_circ):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.converter = Converter()
        self.alignment_path = os.path.join(args_circ.output_folder,
                                           "segemehl_align")
        self.splice_path = os.path.join(args_circ.output_folder,
                                        "segemehl_splice")
        self.candidate_path = os.path.join(args_circ.output_folder,
                                           "circRNA_tables")
        self.gff_folder = os.path.join(args_circ.output_folder, "gffs")
        self.gff_path = os.path.join(args_circ.gffs, "tmp")
        self.splices = {"file": "splicesites.bed",
                        "splice": "splicesites"}
        self.trans = {"file": "transrealigned.bed",
                      "trans": "transrealigned"}
        self.fasta_path = os.path.join(args_circ.fastas, "tmp")

    def _wait_process(self, processes):
        '''wait for the parallels to finish the process'''
        for p in processes:
            p.wait()
            if p.stdout:
                p.stdout.close()
            if p.stdin:
                p.stdin.close()
            if p.stderr:
                p.stderr.close()
            try:
                p.kill()
            except OSError:
                pass
            time.sleep(5)

    def _deal_zip_file(self, read_files):
        tmp_reads = []
        for reads in read_files:
            for read in glob(reads):
                if read.endswith(".bz2"):
                    mod_read = read.replace(".bz2", "")
                    if (".fa" not in mod_read) and (
                            ".fasta" not in mod_read) and (
                            ".fna" not in mod_read):
                        mod_read = mod_read + ".fa"
                    read_out = open(mod_read, "w")
                    tmp_reads.append(mod_read)
                    print(" ".join(["Unzipping", read]))
                    call(["bzcat", read], stdout=read_out)
                    read_out.close()
                elif read.endswith(".gz"):
                    mod_read = read.replace(".gz", "")
                    if (".fa" not in mod_read) and (
                            ".fasta" not in mod_read) and (
                            ".fna" not in mod_read):
                        mod_read = mod_read + ".fa"
                    read_out = open(mod_read, "w")
                    tmp_reads.append(mod_read)
                    print(" ".join(["Unzipping", read]))
                    call(["zcat", read], stdout=read_out)
                    read_out.close()
        return tmp_reads

    def _run_segemehl_fasta_index(self, segemehl_path, fasta_path,
                                  index, fasta):
        call([segemehl_path,
              "-x", os.path.join(fasta_path, index),
              "-d", os.path.join(fasta_path, fasta)])

    def _run_segemehl_align(self, args_circ, index, fasta, read,
                            sam_file, log_file, fasta_prefix):
        out = open(os.path.join(self.alignment_path,
                   fasta_prefix, sam_file), "w")
        log = open(os.path.join(self.alignment_path,
                   fasta_prefix, log_file), "w")
        p = Popen([args_circ.segemehl_path,
                   "-i", os.path.join(self.fasta_path, index),
                   "-d", os.path.join(self.fasta_path, fasta),
                   "-q", read, "-S"],
                  stdout=out, stderr=log)
        return p

    def _align(self, args_circ, read_datas):
        '''align the read. if the bam files are provided, it can be skipped.'''
        prefixs = []
        align_files = []
        for fasta in os.listdir(self.fasta_path):
            index = fasta.replace(".fa", ".idx")
            self._run_segemehl_fasta_index(args_circ.segemehl_path,
                                           self.fasta_path, index, fasta)
            processes = []
            num_process = 0
            fasta_prefix = fasta.replace(".fa", "")
            prefixs.append(fasta_prefix)
            self.helper.check_make_folder(os.path.join(
                                self.alignment_path, fasta_prefix))
            for reads in read_datas:
                for read in reads["files"]:
                    num_process += 1
                    read_name = read.split("/")[-1]
                    if read_name.endswith(".fa") or \
                       read_name.endswith(".fna") or \
                       read_name.endswith("fasta"):
                        filename = read_name.split(".")
                        read_prefix = ".".join(filename[:-1])
                        sam_file = "_".join([read_prefix, fasta_prefix + ".sam"])
                        log_file = "_".join([read_prefix, fasta_prefix + ".log"])
                        align_files.append("_".join([read_prefix, fasta_prefix]))
                        print("Mapping {0}".format(sam_file))
                        p = self._run_segemehl_align(
                                args_circ, index, fasta, read,
                                sam_file, log_file, fasta_prefix)
                        processes.append(p)
                        if num_process == args_circ.cores:
                            self._wait_process(processes)
                            num_process = 0
            self._wait_process(processes)
        return align_files, prefixs

    def _run_samtools_convert_bam(self, samtools_path, pre_sam, out_bam):
        call([samtools_path, "view", "-bS", pre_sam, "-o", out_bam])

    def _convert_sam2bam(self, sub_alignment_path, samtools_path, align_files):
        bam_files = []
        convert_ones = []
        remove_ones = []
        for sam in os.listdir(sub_alignment_path):
            pre_sam = os.path.join(sub_alignment_path, sam)
            if sam.endswith(".sam"):
                bam_file = sam.replace(".sam", ".bam")
                print("Converting {0} to {1}".format(sam, bam_file))
                out_bam = os.path.join(sub_alignment_path, bam_file)
                self._run_samtools_convert_bam(samtools_path, pre_sam, out_bam)
                bam_files.append(out_bam)
                if align_files:
                    if bam_file.replace(".bam", "") not in align_files:
                        convert_ones.append(out_bam)
                    else:
                        remove_ones.append(pre_sam)
            elif sam.endswith(".bam"):
                if (pre_sam not in convert_ones) and (
                        pre_sam not in remove_ones):
                    bam_files.append(pre_sam)
            elif sam.endswith(".log"):
                os.remove(pre_sam)
        return bam_files, convert_ones, remove_ones

    def _run_samtools_merge_sort(self, samtools_path, prefix,
                                 out_folder, bam_datas):
        for bam_data in bam_datas:
            print("Merging bam files for " + bam_data["sample"])
            sample_bam = os.path.join(out_folder, "_".join([
                prefix, bam_data["sample"] + ".bam"]))
            if len(bam_data["files"]) <= 1:
                shutil.copyfile(bam_data["files"][0], sample_bam)
            else:
                file_line = " ".join(bam_data["files"])
                os.system(" ".join([samtools_path, "merge",
                                    sample_bam, file_line]))
            print("Sorting bam files for " + bam_data["sample"])
            sort_sample = os.path.join(out_folder,
                  "_".join([prefix, bam_data["sample"] + "_sort.bam"]))
            call([samtools_path, "sort", "-o", sort_sample, sample_bam])
            os.remove(sample_bam)
            print("Converting bam file to sam file for " + bam_data["sample"])
            call([samtools_path, "view", "-h", "-o",
                  sort_sample.replace(".bam", ".sam"), sort_sample])

    def _merge_sort_aligment_file(
            self, bam_datas, read_datas, samtools_path,
            out_folder, convert_ones, tmp_reads, remove_ones, prefix):
        if bam_datas is None:
            merge_bam_datas = []
            for read_data in read_datas:
                bam_files = []
                for read in read_data["files"]:
                    read_prefix = ".".join(
                        read.split("/")[-1].split(".")[:-1])
                    bam_files.append(os.path.join(
                        self.alignment_path, prefix,
                        "_".join([read_prefix, prefix + ".bam"])))
                merge_bam_datas.append({"sample": read_data["sample"],
                                        "files": bam_files})
        elif (bam_datas is not None) and (read_datas is not None):
            merge_bam_datas = copy.deepcopy(bam_datas)
            for bam_data in merge_bam_datas:
                for read_data in read_datas:
                    if bam_data["sample"] == read_data["sample"]:
                        for read in read_data["files"]:
                            read_prefix = ".".join(
                                read.split("/")[-1].split(".")[:-1])
                            bam = os.path.join(
                                self.alignment_path, prefix,
                                "_".join([read_prefix, prefix + ".bam"]))
                            if (bam not in bam_data["files"]):
                                bam_data["files"].append(bam)
        else:
            merge_bam_datas = copy.deepcopy(bam_datas)
        self._run_samtools_merge_sort(samtools_path, prefix,
                                      out_folder, merge_bam_datas)
        for bam in convert_ones:
            os.remove(bam)
        for sam in remove_ones:
            os.remove(sam)
        if len(tmp_reads) != 0:
            for read in tmp_reads:
                os.remove(read)

    def _run_testrealign(self, prefix, testrealign_path, out_folder):
        sub_splice_path = os.path.join(self.splice_path, prefix)
        if not os.path.exists(sub_splice_path):
            os.mkdir(sub_splice_path)
        err_log = os.path.join(sub_splice_path, prefix + ".log")
        print("Running testrealign.x for {0}".format(prefix))
        for sam_file in os.listdir(out_folder):
            if sam_file.endswith("sort.sam"):
                sample_prefix = sam_file.replace("_sort.sam", "")
                command = " ".join([
                    testrealign_path,
                    "-d", os.path.join(self.fasta_path, prefix + ".fa"),
                    "-q", os.path.join(out_folder, sam_file), "-n",
                    "-U", os.path.join(sub_splice_path,
                                       sample_prefix + "_splicesites.bed"),
                    "-T", os.path.join(sub_splice_path,
                                       sample_prefix + "_transrealigned.bed")])
                os.system(command + " 2>" + err_log)
        self.helper.remove_all_content(out_folder, ".sam", "file")

    def _merge_bed(self, fastas, splice_path, output_folder):
        '''Merge the bed files for analysis'''
        fa_prefixs = []
        for fasta in os.listdir(fastas):
            headers = []
            if (fasta.endswith(".fa") or fasta.endswith(".fna") or
                    fasta.endswith(".fasta")):
                with open(os.path.join(fastas, fasta), "r") as f_h:
                    for line in f_h:
                        line = line.strip()
                        if line.startswith(">"):
                            headers.append(line[1:])
                filename = fasta.split(".")
                fasta_prefix = ".".join(filename[:-1])
                fa_prefixs.append(fasta_prefix)
                bed_folder = os.path.join(
                    output_folder, fasta_prefix)
                self.helper.check_make_folder(bed_folder)
                samples = []
                for header in headers:
                    for splice in os.listdir(os.path.join(
                            splice_path, header)):
                        if splice.endswith(".bed"):
                            if self.splices["file"] in splice:
                                sample = splice.replace(header, "")
                                sample = sample.replace(
                                    self.splices["file"], "")
                                if sample not in samples:
                                    samples.append(sample)
                            shutil.copyfile(
                                os.path.join(
                                splice_path, header, splice),
                                os.path.join(
                                bed_folder, "tmp_" + splice))
                for sample in samples:
                    out_splice = os.path.join(bed_folder, "".join([
                        fasta_prefix + sample + self.splices["file"]]))
                    out_trans = os.path.join(bed_folder, "".join([
                        fasta_prefix + sample + self.trans["file"]]))
                    if os.path.exists(out_splice):
                        os.remove(out_splice)
                    if os.path.exists(out_trans):
                        os.remove(out_trans)
                    for file_ in os.listdir(bed_folder):
                        if (self.splices["splice"] in file_) and (
                                sample in file_):
                            self.helper.merge_file(os.path.join(
                                    bed_folder, file_), out_splice)
                        elif (self.trans["trans"] in file_) and (
                                sample in file_):
                            self.helper.merge_file(os.path.join(
                                    bed_folder, file_), out_trans)
        self.helper.remove_all_content(splice_path, None, "dir")
        return samples, fa_prefixs

    def _stat_and_gen_gff(self, prefixs, samples, args_circ):
        '''do statistics and print the result to gff file'''
        for prefix in prefixs:
            self.helper.check_make_folder(os.path.join(self.gff_folder,
                                                       prefix))
            self.helper.check_make_folder(os.path.join(self.splice_path,
                                                       prefix))
            for bed in os.listdir(os.path.join(
                args_circ.output_folder, prefix)):
                if (bed.split("_")[0] != "tmp") and (bed.endswith(".bed")):
                    shutil.copy(
                        os.path.join(args_circ.output_folder, prefix, bed),
                        os.path.join(self.splice_path, prefix))
            self.helper.check_make_folder(os.path.join(
                                          self.candidate_path, prefix))
            print("Comparing with annotation of {0}".format(prefix))
            for sample in samples:
                splice_file = os.path.join(
                    self.splice_path, prefix,
                    "".join([prefix, sample, self.splices["file"]]))
                detect_circrna(splice_file, os.path.join(
                               self.gff_path, prefix + ".gff"),
                               os.path.join(self.candidate_path, prefix,
                               "".join([prefix, sample, "circRNA_all.csv"])),
                               args_circ, os.path.join(args_circ.stat_folder,
                               "".join(["stat_", prefix, sample,
                                        "circRNA.csv"])))
                self.converter.convert_circ2gff(
                     os.path.join(self.candidate_path, prefix,
                                  "".join([prefix, sample, "circRNA_all.csv"])),
                     args_circ, os.path.join(
                                self.gff_folder, prefix,
                                "".join([prefix, sample, "circRNA_all.gff"])),
                     os.path.join(self.gff_folder, prefix,
                                  "".join([prefix, sample, "circRNA_best.gff"])))

    def _extract_input_files(self, inputs):
        input_datas = []
        for input_ in inputs:
            datas = input_.split(":")
            if len(datas) != 2:
                print("Error: the format of --bam_files or "
                      "--read_files is wrong!")
                sys.exit()
            for file_ in datas[-1].split(","):
                if not os.path.exists(file_):
                    print("Error: some files in -bam_files or "
                          "--read_files are not exist!")
                    sys.exit()
            input_datas.append({"sample": datas[0],
                                "files": datas[-1].split(",")})
        return input_datas

    def _combine_read_bam(self, bam_files, bam_datas, read_datas):
        if bam_datas is not None:
            for bam_data in bam_datas:
                for read_data in read_datas:
                    if bam_data["sample"] == read_data["sample"]:
                        for read in read_data["files"]:
                            prefix = ".".join(
                                read.split("/")[-1].split(".")[:-1])
                            bam = os.path.join(self.alignment_path,
                                               prefix + ".bam")
                            if (bam in bam_files) and (
                                    bam not in bam_data["files"]):
                                bam_data["files"].append(bam)
        else:
            bam_datas = []
            for read_data in read_datas:
                bam_files = []
                for read in read_data["files"]:
                    prefix = ".".join(
                        read.split("/")[-1].split(".")[:-1])
                    bam_files.append(os.path.join(
                        self.alignment_path, prefix + ".bam"))
                bam_datas.append({"sample": read_data["sample"],
                                  "files": bam_files})
        return bam_datas

    def _remove_tmp_files(self, args_circ, fa_prefixs):
        self.helper.remove_tmp_dir(args_circ.fastas)
        self.helper.remove_tmp_dir(args_circ.gffs)
        self.helper.remove_all_content(args_circ.output_folder,
                                       ".bam", "file")
        for prefix in fa_prefixs:
            shutil.rmtree(os.path.join(args_circ.output_folder, prefix))

    def run_circrna(self, args_circ):
        '''detection of circRNA'''
        bam_datas = None
        read_datas = None
        if (args_circ.bams is None) and (args_circ.read_files is None):
            print("Error: --bam_files or --read_files should be assigned.")
            sys.exit()
        if args_circ.bams is not None:
            bam_datas = self._extract_input_files(args_circ.bams)
        if args_circ.read_files is not None:
            read_datas = self._extract_input_files(args_circ.read_files)
        for gff in os.listdir(args_circ.gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(
                                                 args_circ.gffs, gff))
        if args_circ.segemehl_path is None:
            print("Error: please assign segemehl folder!!")
            sys.exit()
        self.multiparser.parser_fasta(args_circ.fastas)
        self.multiparser.parser_gff(args_circ.gffs, None)
        self.multiparser.combine_gff(args_circ.fastas, self.gff_path,
                                     "fasta", None)
        tmp_reads = []
        if args_circ.read_files:
            tmp_reads = self._deal_zip_file(args_circ.read_files)
            align_files, prefixs = self._align(args_circ, read_datas)
        else:
            align_files = None
        prefixs = []
        for fasta in os.listdir(self.fasta_path):
            if fasta.endswith(".fa"):
                fasta_prefix = fasta.replace(".fa", "")
                prefixs.append(fasta_prefix)
        for prefix in prefixs:
            if args_circ.read_files:
                sub_alignment_path = os.path.join(self.alignment_path, prefix)
                bam_files, convert_ones, remove_ones = self._convert_sam2bam(
                    sub_alignment_path, args_circ.samtools_path, align_files)
            else:
                convert_ones = []
                remove_ones = []
            self._merge_sort_aligment_file(
                bam_datas, read_datas, args_circ.samtools_path,
                args_circ.output_folder,
                convert_ones, tmp_reads, remove_ones, prefix)
            self._run_testrealign(prefix, args_circ.testrealign_path,
                                  args_circ.output_folder)
        samples, fa_prefixs = self._merge_bed(
            args_circ.fastas, self.splice_path, args_circ.output_folder)
        self._stat_and_gen_gff(fa_prefixs, samples, args_circ)
        self._remove_tmp_files(args_circ, fa_prefixs)



import os
import shutil
from annogesiclib.helper import Helper
from annogesiclib.multiparser import Multiparser
from annogesiclib.gene_ontology import retrieve_uniprot, map2goslim


class GoTermFinding(object):
    '''Retrieving the GO term'''

    def __init__(self, args_go):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.out_all = os.path.join(args_go.out_folder, "all_CDS")
        self.out_express = os.path.join(args_go.out_folder, "expressed_CDS")
        self.result_all_path = os.path.join(self.out_all, "GO_term_results")
        self.result_express_path = os.path.join(self.out_express,
                                                "GO_term_results")
        self.gff_path = os.path.join(args_go.gffs, "tmp")
        if args_go.trans is not None:
            self.tran_path = os.path.join(args_go.trans, "tmp")
        else:
            self.tran_path = None
        self.stat_all_path = os.path.join(self.out_all, "statistics")
        self.stat_express_path = os.path.join(self.out_express,
                                              "statistics")
        self.all_strain = "all_strains_uniprot.csv"

    def _retrieve_go(self, uniprot, out_path, type_):
        prefixs = []
        for gff in os.listdir(self.gff_path):
            prefix = gff.replace(".gff", "")
            prefixs.append(prefix)
            self.helper.check_make_folder(os.path.join(out_path, prefix))
            out_file = os.path.join(out_path, prefix,
                                    "_".join([prefix, "uniprot.csv"]))
            print("Extracting GO terms of {0} from UniProt".format(prefix))
            if self.tran_path is not None:
                tran_file = os.path.join(self.tran_path,
                                         "_".join([prefix, "transcript.gff"]))
            else:
                tran_file = None
            retrieve_uniprot(uniprot, os.path.join(self.gff_path, gff),
                             out_file, tran_file, type_)

    def _merge_files(self, gffs, out_path, out_folder):
        '''merge the files according to the input genome folder'''
        folders = []
        for folder in os.listdir(gffs):
            if folder.endswith("gff_folder"):
                folder_prefix = folder.replace(".gff_folder", "")
                folder_path = os.path.join(out_folder, folder_prefix)
                self.helper.check_make_folder(folder_path)
                folders.append(folder_path)
                filenames = []
                for gff in os.listdir(os.path.join(gffs, folder)):
                    if gff.endswith(".gff"):
                        filenames.append(gff.replace(".gff", ""))
                out_all = os.path.join(folder_path, self.all_strain)
                if len(filenames) > 1:
                    if self.all_strain in os.listdir(folder_path):
                        os.remove(out_all)
                    for filename in filenames:
                        csv_file = "_".join([filename, "uniprot.csv"])
                        self.helper.merge_file(os.path.join(out_path,
                                               filename, csv_file), out_all)
                        shutil.copy(os.path.join(out_path, filename, csv_file),
                                    folder_path)
                else:
                    shutil.copyfile(os.path.join(out_path, filenames[0],
                                    "_".join([filenames[0], "uniprot.csv"])),
                                    out_all)
        self.helper.remove_all_content(out_path, None, "dir")
        self.helper.remove_all_content(out_path, None, "file")
        for folder in folders:
            folder_prefix = folder.split("/")[-1]
            shutil.move(folder, os.path.join(out_path, folder_prefix))

    def _stat(self, out_path, stat_path, go, goslim, out_folder):
        for folder in os.listdir(out_path):
            strain_stat_path = os.path.join(stat_path, folder)
            self.helper.check_make_folder(strain_stat_path)
            fig_path = os.path.join(strain_stat_path, "figs")
            if "fig" not in os.listdir(strain_stat_path):
                os.mkdir(fig_path)
            print("Computing statistics of {0}".format(folder))
            map2goslim(goslim, go,
                       os.path.join(out_path, folder, self.all_strain),
                       os.path.join(strain_stat_path,
                                    "_".join(["stat", folder + ".csv"])),
                       out_folder)
            self.helper.move_all_content(out_folder, fig_path,
                                         ["_three_roots.png"])
            self.helper.move_all_content(out_folder, fig_path,
                                         ["_molecular_function.png"])
            self.helper.move_all_content(out_folder, fig_path,
                                         ["_cellular_component.png"])
            self.helper.move_all_content(out_folder, fig_path,
                                         ["_biological_process.png"])

    def run_go_term(self, args_go):
        for gff in os.listdir(args_go.gffs):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(
                                                 args_go.gffs, gff))
        self.multiparser.parser_gff(args_go.gffs, None)
        if args_go.trans is not None:
            self.multiparser.parser_gff(args_go.trans, "transcript")
        print("Computing all CDS")
        self._retrieve_go(args_go.uniprot, self.result_all_path, "all")
        self._merge_files(args_go.gffs, self.result_all_path, self.out_all)
        self._stat(self.result_all_path, self.stat_all_path, args_go.go,
                   args_go.goslim, self.out_all)
        if args_go.trans is not None:
            print("Computing express CDS")
            self._retrieve_go(args_go.uniprot, self.result_express_path,
                              "express")
            self._merge_files(args_go.gffs, self.result_express_path,
                              self.out_express)
            self._stat(self.result_express_path, self.stat_express_path,
                       args_go.go, args_go.goslim, self.out_express)
        self.helper.remove_tmp_dir(args_go.gffs)
        if args_go.trans is not None:
            self.helper.remove_tmp_dir(args_go.trans)



import os
import sys
import copy
import shutil
import csv
from Bio.Seq import Seq
from Bio.Alphabet import generic_dna
from annogesiclib.gff3 import Gff3Parser


class Helper(object):
    '''For some small and regular modules for ANNOgesic'''

    def __init__(self):
        self.gff3parser = Gff3Parser()

    def get_strand_name(self, strand):
        '''change the strand name to f/r'''
        name = ""
        if strand == "+":
            name = "f"
        else:
            name = "r"
        return name

    def _fix_break_line(self, tar, prefix):
        '''fix the break line at the end of file'''
        tmp_out = open("tmp_file", "w")
        first = True
        with open(tar) as fh:
            for line in fh:
                line = line.strip()
                if (prefix == ">"):
                    if (prefix in line) and (first):
                        first = False
                    elif (prefix in line) and (not first) and (
                          not line.startswith(prefix)):
                        line = line.replace(prefix, "\n" + prefix)
                else:
                    row = line.split("\t")
                    if (len(row) > 9):
                        for strain in prefix:
                            if strain in line:
                                line = line.replace(strain, "\n" + strain)
                                break
                tmp_out.write(line + "\n")
        tmp_out.close()
        os.remove(tar)
        shutil.move("tmp_file", tar)

    def merge_file(self, ref, tar):
        '''merge two files'''
        os.system(" ".join(["cat", ref, ">>", tar]))
        if tar.endswith(".fa"):
            self._fix_break_line(tar, ">")
        elif tar.endswith(".gff"):
            strains = []
            fh = open(ref, "r")
            for row in csv.reader(fh, delimiter='\t'):
                if row[0] not in strains:
                    strains.append(row[0])
            fh.close()
            self._fix_break_line(tar, strains)

    def merge_blast_out(self, ref, tar):
        tmp_out = tar + "_tmp"
        out = open(tmp_out, "w")
        file_num = 0
        for file_ in (ref, tar):
            start = False
            finish = False
            with open(file_) as fh:
                for line in fh:
                    check_line = line.strip()
                    if (check_line.startswith("Query")):
                        start = True
                    if (not start) and (not finish) and (file_num == 0):
                        out.write(line)
                    if start and (not check_line.startswith("Database")):
                        out.write(line)
                    if (check_line.startswith("Database")) and start:
                        if file_num == 1:
                            out.write(line)
                        else:
                            start = False
                            finish = True
            file_num += 1
        os.remove(tar)
        os.remove(ref)
        shutil.move(tmp_out, tar)

    def remove_all_content(self, folder, feature, data_type):
        '''remove all files in one folder'''
        for file_ in os.listdir(folder):
            remove = False
            if feature is None:
                remove = True
            else:
                if feature in file_:
                    remove = True
            if remove:
                target = os.path.join(folder, file_)
                if (data_type == "file") and os.path.isfile(target):
                    os.remove(target)
                elif (data_type == "dir") and os.path.isdir(target):
                    shutil.rmtree(target)

    def move_all_content(self, ref_folder, tar_folder, features):
        '''move all files form one folder to another one'''
        for file_ in os.listdir(ref_folder):
            move = False
            if (features is not None):
                move = True
                for feature in features:
                    if (feature not in file_):
                        move = False
            elif (features is None):
                move = True
            if move:
                shutil.move(os.path.join(ref_folder, file_),
                            os.path.join(tar_folder, file_))

    def remove_tmp(self, folder):
        '''remove the tmp folder'''
        if folder:
            if os.path.isdir(os.path.join(folder, "tmp")):
                shutil.rmtree(os.path.join(folder, "tmp"))
            self.remove_all_content(folder, "_folder", "dir")

    def remove_tmp_dir(self, folder):
        if folder is not None:
            if os.path.isdir(folder):
                shutil.rmtree(folder)

    def remove_tmp_dir_ori(self, ori_folder, folder, types):
        if folder is not None:
            if os.path.isdir(folder):
                for file_ in os.listdir(ori_folder):
                    for type_ in types:
                        ori_file = os.path.join(ori_folder, file_)
                        if (file_.endswith(type_)) and (
                                os.path.isfile(ori_file)):
                            shutil.move(ori_file, ori_file + "_old")
                for file_ in os.listdir(folder):
                    if os.path.isfile(os.path.join(folder, file_)):
                        shutil.move(os.path.join(folder, file_), ori_folder)
                shutil.rmtree(folder)

    def remove_wigs(self, wigs):
        '''remove the merge wig folder which is generated by ANNOgesic'''
        if wigs:
            folder = wigs.split("/")
            folder = "/".join(folder[:-1])
            if os.path.isdir(os.path.join(folder, "merge_wigs")):
                shutil.rmtree(os.path.join(folder, "merge_wigs"))
        self.remove_tmp(wigs)

    def get_correct_file(self, datas, feature, prefix, for_wig_type, libs):
        '''get the correct file by comparing the strain name'''
        detect = False
        for data in os.listdir(datas):
            if os.path.isfile(os.path.join(datas, data)):
                if for_wig_type is None:
                    if feature in data:
                        file_ = data[:-1 * len(feature)]
                        if prefix == file_:
                            detect = True
                            return os.path.join(datas, data)
                else:
                    filename = data.split("_STRAIN_")
                    if ("reverse" in data) and ("forward" in data):
                        print("Error: Assign reverse or forward wigs!!!")
                        sys.exit()
                    elif (prefix == filename[-1][:-1 * len(feature)]):
                        if (for_wig_type == "forward"):
                            for lib in libs:
                                if (filename[0] in lib) and (lib[-1] == "+"):
                                    return os.path.join(datas, data)
                        if (for_wig_type == "reverse"):
                            for lib in libs:
                                if (filename[0] in lib) and (lib[-1] == "-"):
                                    return os.path.join(datas, data)
        if detect:
            detect = False
        else:
            print("Warning: No proper file - " + prefix + feature)
            return None

    def check_make_folder(self, folder):
        '''make new folder (if the folder exists, 
        it will remove it and create new one)'''
        path = "/".join(folder.split("/")[:-1])
        gen_folder = folder.split("/")[-1]
        if gen_folder in os.listdir(path):
            shutil.rmtree(os.path.join(path, gen_folder))
        os.mkdir(os.path.join(path, gen_folder))

    def sort_gff(self, gff_file, out_file):
        gffs = []
        g_f = open(gff_file, "r")
        for entry in self.gff3parser.entries(g_f):
            gffs.append(entry)
        g_f.close()
        sort_gffs = sorted(gffs, key=lambda x: (x.seq_id, x.start,
                                                x.end, x.strand))
        out = open(out_file, "w")
        out.write("##gff-version 3\n")
        for gff in sort_gffs:
            out.write("\t".join([str(field) for field in [
                        gff.seq_id, gff.source, gff.feature, gff.start,
                        gff.end, gff.score, gff.strand, gff.phase,
                        gff.attribute_string]]) + "\n")
        out.close()

    def extract_gene(self, seq, start, end, strand):
        '''extract gene seqence'''
        fasta = ''
        if strand == "+":
            return seq[(int(start)-1):int(end)]
        else:
            rev_seq = seq[(int(start)-1):int(end)]
            fasta = self._reverse_seq(rev_seq)
            return fasta

    def _reverse_seq(self, rev_seq):
        '''deal with the reverse strand'''
        fasta = ""
        for base in rev_seq[::-1]:
            if base.upper() == 'A':
                fasta = fasta + 'T'
            elif base.upper() == 'T':
                fasta = fasta + 'A'
            elif base.upper() == 'C':
                fasta = fasta + 'G'
            elif base.upper() == 'G':
                fasta = fasta + 'C'
        return fasta

    def _add_element(self, list_, type_, gff):
        if type_ in gff.attributes.keys():
            list_.add(gff.attributes[type_])

    def check_uni_attributes(self, gff_file):
        '''check the attributes of gff filee, ie ID has to be unique'''
        print("Checking gff file of {0}".format(gff_file.split("/")[-1]))
        gffs = []
        fh = open(gff_file)
        for entry in self.gff3parser.entries(fh):
            if (entry.feature == "source") or (
                    entry.feature == "region"):
                length = entry.end
            else:
                length = None
            gffs.append(entry)
        gffs = sorted(gffs, key=lambda x: (x.seq_id, x.start, x.end, x.strand))
        first = True
        ids = set()
        locus_tags = set()
        pre_gff = None
        for gff in gffs:
            if (gff.feature != "source") and (
                    gff.feature != "region"):
                if length is not None:
                    if gff.end > length:
                        name = "".join([gff.feature, ":", str(gff.start), "-",
                                        str(gff.end), "_", gff.strand])
                        print("Error: The end point of " + name +
                              " is longer than the length of whole genome.")
                        print("Please check the gff file.")
                        sys.exit()
            if first:
                first = False
                self._add_element(ids, "ID", gff)
                self._add_element(locus_tags, "locus_tag", gff)
            else:
                if gff.seq_id == pre_gff.seq_id:
                    if "ID" in gff.attributes.keys():
                        if gff.attributes["ID"] in ids:
                            print("Warninng: Repeat ID {0} "
                                  "in gff file!!!".format(
                                      gff.attributes["ID"]))
                        else:
                            self._add_element(ids, "ID", gff)
                    if "locus_tag" in gff.attributes.keys():
                        if gff.attributes["locus_tag"] in ids:
                            print("Warning: Repeat locus_tag {0} "
                                  "in gff file!!!".format(
                                      gff.attributes["locus_tag"]))
                        else:
                            self._add_element(locus_tags, "locus_tag", gff)
            pre_gff = copy.copy(gff)
        fh.close()

    def _read_fasta(self, fasta_file):
        seq = ""
        with open(fasta_file, "r") as seq_f:
            for line in seq_f:
                line = line.strip()
                if line.startswith(">"):
                    continue
                else:
                    seq = seq + line
        return seq

    def get_seq(self, gff_file, fasta_file, out_file):
        gff_f = open(gff_file, "r")
        out = open(out_file, "w")
        seq = self._read_fasta(fasta_file)
        num = 0
        for entry in self.gff3parser.entries(gff_f):
            gene = self.extract_gene(seq, entry.start, entry.end, entry.strand)
            if "ID" in entry.attributes.keys():
                id_ = entry.attributes["ID"]
            else:
                id_ = entry.feature + str(num)
            out.write(">{0}|{1}|{2}|{3}|{4}\n{5}\n".format(
                      id_, entry.seq_id, entry.start,
                      entry.end, entry.strand, gene))
            num += 1
        gff_f.close()
        out.close()

    def get_cds_seq(self, gff_file, fasta_file, out_file):
        seq = self._read_fasta(fasta_file)
        out = open(out_file, "w")
        cdss = []
        gh = open(gff_file)
        for entry in self.gff3parser.entries(gh):
            if entry.feature == "CDS":
                cdss.append(entry)
        cdss = sorted(cdss, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
        for entry in cdss:
            cds = self.extract_gene(seq, entry.start, entry.end, entry.strand)
            if "protein_id" in entry.attributes.keys():
                protein_id = entry.attributes["protein_id"]
            elif "locus_tag" in entry.attributes.keys():
                protein_id = entry.attributes["locus_tag"]
            else:
                protein_id = entry.attributes["ID"]
            out.write("_".join([">" + entry.seq_id, "_" + protein_id,
                      entry.strand, str(entry.start), str(entry.end)]) + "\n")
            out.write(cds + "\n")
        out.close()
        gh.close()

    def translation(self, dna_file, protein_file):
        '''translate the DNA to residues'''
        out = open(protein_file, "w")
        with open(dna_file) as d_h:
            for seq in d_h:
                seq = seq.strip()
                if seq.startswith(">"):
                    out.write(seq + "\n")
                else:
                    dna = Seq(seq, generic_dna)
                    out.write(str(dna.translate()) + "\n")
        out.close()



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def read_gff(gff_file):
    datas = []
    for entry in Gff3Parser().entries(open(gff_file)):
        datas.append(entry)
    datas = sorted(datas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return datas


def print_table(srna_table, out_t, srnas):
    fh = open(srna_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        for srna in srnas:
            if (row[0] == srna.seq_id) and (
                int(row[2]) == srna.start) and (
                int(row[3]) == srna.end) and (
                    row[4] == srna.strand):
                if "with_term" in srna.attributes.keys():
                    with_term = [srna.attributes["with_term"]]
                else:
                    with_term = ["NA"]
                out_t.write("\t".join(row + with_term) + "\n")


def compare_srna_term(srna_gff, srna_table, term_file, fuzzy_b, fuzzy_a):
    '''Comparison of sRNA and terminator. 
    It can search the sRNA which is associated with terminator'''
    srnas = read_gff(srna_gff)
    terms = read_gff(term_file)
    out_g = open("tmp_srna.gff", "w")
    out_t = open("tmp_srna.csv", "w")
    out_g.write("##gff-version 3\n")
    for srna in srnas:
        detect = False
        for term in terms:
            if (srna.seq_id == term.seq_id) and (
                    srna.strand == term.strand):
                if (srna.strand == "+"):
                    if (((srna.end - term.end) <= fuzzy_b) and (
                            srna.end >= term.end) and (
                            srna.start < term.start)) or (
                            ((term.start - srna.end) <= fuzzy_a) and (
                             term.start >= srna.end)) or (
                            (srna.end > term.start) and (
                             srna.end < term.end) and (
                             srna.start < term.start)):
                        term_string = (term.feature + ":" + str(term.start) +
                                       "-" + str(term.end) + "_" + term.strand)
                        srna.attributes["with_term"] = term_string
                        detect = True
                        break
                else:
                    if (((term.start - srna.start) <= fuzzy_b) and (
                            term.start >= srna.start) and (
                            term.end < srna.end)) or (
                            ((srna.start - term.end) <= fuzzy_a) and (
                             srna.start >= term.end)) or (
                            (srna.start > term.start) and (
                             srna.start < term.end) and (
                             srna.end > term.end)):
                        term_string = (term.feature + ":" + str(term.start) +
                                       "-" + str(term.end) + "_" + term.strand)
                        srna.attributes["with_term"] = term_string
                        detect = True
                        break
        if "end_cleavage" in srna.attributes.keys():
            if (srna.attributes["end_cleavage"] != "NA") and (
                    "with_term" not in srna.attributes.keys()):
                srna.attributes["with_term"] = srna.attributes["end_cleavage"]
            elif (srna.attributes["end_cleavage"] != "NA") and (
                    "with_term" in srna.attributes.keys()):
                srna.attributes["with_term"] = ",".join([
                    srna.attributes["with_term"],
                    srna.attributes["end_cleavage"]])
        if detect:
            out_g.write(srna.info + ";with_term=" +
                        srna.attributes["with_term"] + "\n")
        else:
            out_g.write(srna.info + ";with_term=NA" + "\n")
    print_table(srna_table, out_t, srnas)
    os.remove(srna_gff)
    os.remove(srna_table)
    out_t.close()
    out_g.close()
    shutil.move("tmp_srna.gff", srna_gff)
    shutil.move("tmp_srna.csv", srna_table)



import os
import sys
import shutil
from subprocess import call
from annogesiclib.helper import Helper
from annogesiclib.multiparser import Multiparser
from annogesiclib.converter import Converter
from annogesiclib.get_inter_seq import intergenic_seq
from annogesiclib.extract_sec_info import extract_info_sec
from annogesiclib.get_polyT import poly_t
from annogesiclib.detect_coverage_term import detect_coverage
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.stat_term import stat_term
from annogesiclib.compare_tran_term import compare_term_tran


class Terminator(object):
    '''detection of terminator'''

    def __init__(self, args_term):
        self.multiparser = Multiparser()
        self.helper = Helper()
        self.converter = Converter()
        self.gff_parser = Gff3Parser()
        self.gff_path = os.path.join(args_term.gffs, "tmp")
        self.fasta_path = os.path.join(args_term.fastas, "tmp")
        self.tran_path = os.path.join(args_term.trans, "tmp")
        self.outfolder = {"term": os.path.join(args_term.out_folder, "gffs"),
                          "csv": os.path.join(args_term.out_folder, "tables")}
        self.terms = {"all": os.path.join(self.outfolder["term"],
                                          "all_candidates"),
                      "express": os.path.join(self.outfolder["term"],
                                              "express"),
                      "best": os.path.join(self.outfolder["term"], "best"),
                      "non": os.path.join(self.outfolder["term"],
                                          "non_express")}
        self.csvs = {"all": os.path.join(self.outfolder["csv"],
                                         "all_candidates"),
                     "express": os.path.join(self.outfolder["csv"], "express"),
                     "best": os.path.join(self.outfolder["csv"], "best"),
                     "non": os.path.join(self.outfolder["csv"], "non_express")}
        self.combine_path = os.path.join(self.gff_path, "combine")
        self.tmps = {"transterm": os.path.join(os.getcwd(), "tmp_transterm"),
                     "hp": "transtermhp", "hp_gff": "transtermhp.gff",
                     "hp_path": "tmp_transterm/tmp",
                     "term_table": os.path.join(os.getcwd(), "tmp_term_table"),
                     "merge": os.path.join(os.getcwd(), "tmp_merge_gff"),
                     "gff": "tmp.gff",
                     "folder": os.path.join(os.getcwd(), "tmp")}
        self.suffixs = {"gff": "term.gff", "csv": "term.csv",
                        "allgff": "term_all.gff"}
        if args_term.srnas:
            self.srna_path = os.path.join(args_term.srnas, "tmp")
        else:
            self.srna_path = None
        self._make_gff_folder()

    def _combine_annotation(self, combine_file, files):
        with open(combine_file, 'w') as result:
            for file_ in files:
                check_start = False
                fh = open(file_, 'r')
                for line in fh:
                    if check_start:
                        result.write(line)
                    if "Location" in line:
                        check_start = True
                if "\n" not in line:
                    result.write("\n")
                fh.close()

    def _make_gff_folder(self):
        self.helper.check_make_folder(self.terms["all"])
        self.helper.check_make_folder(self.csvs["all"])
        self.helper.check_make_folder(self.terms["best"])
        self.helper.check_make_folder(self.csvs["best"])
        self.helper.check_make_folder(self.terms["express"])
        self.helper.check_make_folder(self.csvs["express"])
        self.helper.check_make_folder(self.terms["non"])
        self.helper.check_make_folder(self.csvs["non"])

    def _convert_gff2rntptt(self, gff_path, fasta_path, sRNAs):
        file_types = {}
        prefixs = []
        for gff in os.listdir(gff_path):
            if gff.endswith(".gff"):
                filename = gff.split("/")
                prefix = filename[-1][:-4]
                prefixs.append(prefix)
                gff_file = os.path.join(gff_path, gff)
                rnt_file = os.path.join(gff_path, gff.replace(".gff", ".rnt"))
                ptt_file = os.path.join(gff_path, gff.replace(".gff", ".ptt"))
                fasta = self.helper.get_correct_file(
                             fasta_path, ".fa", prefix, None, None)
                if not fasta:
                    print("Error: No proper file - {0}.fa".format(prefix))
                    sys.exit()
                if sRNAs:
                    self.multiparser.parser_gff(sRNAs, "sRNA")
                    srna = self.helper.get_correct_file(
                            self.srna_path, "_sRNA.gff", prefix, None, None)
                    if (srna) and (fasta):
                        self.converter.convert_gff2rntptt(
                            gff_file, fasta, ptt_file, rnt_file, srna,
                            srna.replace(".gff", ".rnt"))
                        file_types[prefix] = "srna"
                    if (not srna) and (fasta):
                        self.converter.convert_gff2rntptt(
                            gff_file, fasta, ptt_file, rnt_file, None, None)
                        file_types[prefix] = "normal"
                else:
                    self.converter.convert_gff2rntptt(
                        gff_file, fasta, ptt_file, rnt_file, None, None)
                    file_types[prefix] = "normal"
        return file_types, prefixs

    def _combine_ptt_rnt(self, gff_path, file_types, srna_path):
        self.helper.check_make_folder(self.combine_path)
        for prefix, file_type in file_types.items():
            combine_file = os.path.join(self.combine_path, prefix + '.ptt')
            if file_type == "normal":
                files = [os.path.join(gff_path, prefix + ".ptt"),
                         os.path.join(gff_path, prefix + ".rnt")]
                self._combine_annotation(combine_file, files)
            elif file_type == "srna":
                files = [os.path.join(gff_path, prefix + ".ptt"),
                         os.path.join(gff_path, prefix + ".rnt"),
                         os.path.join(srna_path,
                                      "_".join([prefix, "sRNA.rnt"]))]
                self._combine_annotation(combine_file, files)

    def _TransTermHP(self, fasta, file_, out_path, prefix, out, args_term):
        call([args_term.TransTermHP_path, "-p", args_term.expterm_path,
              fasta, os.path.join(self.combine_path, file_), "--t2t-perf",
              os.path.join(out_path, "_".join([
                  prefix,
                  "terminators_within_robust_tail-to-tail_regions.t2t"])),
              "--bag-output", os.path.join(out_path, "_".join([
                  prefix, "best_terminator_after_gene.bag"]))],
             stdout=out)

    def _run_TransTermHP(self, args_term):
        self.helper.check_make_folder(self.tmps["transterm"])
        for file_ in os.listdir(self.combine_path):
            if ".ptt" in file_:
                prefix = file_.replace(".ptt", "")
                fasta = self.helper.get_correct_file(
                             self.fasta_path, ".fa", prefix, None, None)
                if not fasta:
                    print("Error: No proper file - {0}.fa".format(prefix))
                    sys.exit()
                out_path = os.path.join(args_term.hp_folder, prefix)
                self.helper.check_make_folder(out_path)
                out = open(os.path.join(out_path,
                           "_".join([prefix, "terminators.txt"])), "w")
                self._TransTermHP(fasta, file_, out_path,
                                  prefix, out, args_term)
                out.close()
        shutil.rmtree(self.combine_path)

    def _convert_to_gff(self, prefixs, args_term):
        for prefix in prefixs:
            for folder in os.listdir(args_term.hp_folder):
                if prefix == folder:
                    out_path = os.path.join(args_term.hp_folder, folder)
                    for file_ in os.listdir(out_path):
                        if file_.endswith(".bag"):
                            out_file = os.path.join(
                                    self.tmps["transterm"],
                                    "_".join([prefix, self.tmps["hp_gff"]]))
                            self.converter.convert_transtermhp2gff(
                                 os.path.join(out_path, file_), out_file)
        self.multiparser.combine_gff(args_term.gffs, self.tmps["transterm"],
                                     None, self.tmps["hp"])

    def _combine_wigs(self, args_term):
        if (args_term.tex_wigs is not None) and (
                args_term.frag_wigs is not None):
            folder = args_term.tex_wigs.split("/")
            folder = "/".join(folder[:-1])
            merge_wigs = os.path.join(folder, "merge_wigs")
            self.helper.check_make_folder(merge_wigs)
            for wig in os.listdir(args_term.tex_wigs):
                if os.path.isdir(os.path.join(args_term.tex_wigs, wig)):
                    pass
                else:
                    shutil.copy(os.path.join(args_term.tex_wigs, wig),
                                merge_wigs)
            for wig in os.listdir(args_term.frag_wigs):
                if os.path.isdir(os.path.join(args_term.frag_wigs, wig)):
                    pass
                else:
                    shutil.copy(os.path.join(args_term.frag_wigs, wig),
                                merge_wigs)
        elif (args_term.tex_wigs is not None):
            merge_wigs = args_term.tex_wigs
        elif (args_term.frag_wigs is not None):
            merge_wigs = args_term.frag_wigs
        else:
            print("Error: No proper wig files!!!")
            sys.exit()
        return merge_wigs

    def _merge_sRNA(self, sRNAs, prefixs, gff_path):
        '''searching the terminator with sRNA information'''
        if sRNAs is not None:
            self.multiparser.parser_gff(sRNAs, "sRNA")
            self.helper.check_make_folder(self.tmps["merge"])
            for prefix in prefixs:
                tmp_gff = os.path.join(self.tmps["merge"], self.tmps["gff"])
                if self.tmps["gff"] in os.listdir(self.tmps["merge"]):
                    os.remove(tmp_gff)
                self.helper.merge_file(os.path.join(gff_path, prefix + ".gff"),
                                       tmp_gff)
                self.helper.merge_file(os.path.join(
                    self.srna_path, "_".join([prefix, "sRNA.gff"])), tmp_gff)
                self.helper.sort_gff(tmp_gff, os.path.join(
                    self.tmps["merge"], prefix + ".gff"))
                os.remove(tmp_gff)
            merge_path = self.tmps["merge"]
        else:
            merge_path = gff_path
        return merge_path

    def _move_file(self, term_outfolder, csv_outfolder):
        for gff in os.listdir(term_outfolder):
            if gff.endswith("_term.gff"):
                self.helper.sort_gff(os.path.join(term_outfolder, gff),
                                     self.tmps["gff"])
                shutil.move(self.tmps["gff"],
                            os.path.join(term_outfolder, gff))
                prefix = gff.replace("_term.gff", "")
                new_gff = os.path.join(self.terms["all"], "_".join([
                        prefix, self.suffixs["allgff"]]))
                csv_file = os.path.join(
                        os.path.join(self.csvs["all"], "_".join([
                            prefix, self.suffixs["csv"]])))
                out = open(new_gff, "w")
                out.write("##gff-version 3\n")
                out.close()
                self.helper.merge_file(
                        os.path.join(term_outfolder, gff),
                        os.path.join(
                            self.terms["all"], "_".join([
                                prefix, self.suffixs["allgff"]])))
                os.remove(os.path.join(term_outfolder, gff))
                pre_strain = ""
                if ("_".join([prefix, self.suffixs["csv"]]) in
                        os.listdir(self.csvs["all"])):
                    os.remove(csv_file)
                out_csv = open(csv_file, "w")
                out_csv.write("\t".join(["strain", "name", "start", "end",
                              "strand", "detect", "coverage_decrease",
                              "coverage_detail"]) + "\n")
                out_csv.close()
                fh = open(new_gff)
                for entry in self.gff_parser.entries(fh):
                    if entry.seq_id != pre_strain:
                        self.helper.merge_file(os.path.join(
                            self.tmps["term_table"], "_".join([
                                entry.seq_id, "term_raw.csv"])),
                            os.path.join(self.csvs["all"], "_".join([
                                prefix, self.suffixs["csv"]])))
                    pre_strain = entry.seq_id
                fh.close()

    def _run_rnafold(self, RNAfold_path, tmp_seq, tmp_sec, prefix):
        print("Computing secondray structure of {0}".format(prefix))
        self.helper.check_make_folder(self.tmps["folder"])
        pre_cwd = os.getcwd()
        os.chdir(self.tmps["folder"])
        os.system(" ".join([RNAfold_path, "<", os.path.join("..", tmp_seq),
                  ">", os.path.join("..", tmp_sec)]))
        os.chdir(pre_cwd)
        shutil.rmtree(self.tmps["folder"])

    def _compute_intersection_forward_reverse(
            self, prefixs, merge_path, wig_path, merge_wigs, args_term):
        '''the approach for searching gene converged region terminator'''
        for prefix in prefixs:
            tmp_seq = os.path.join(args_term.out_folder,
                                   "_".join(["inter_seq", prefix]))
            tmp_index = os.path.join(args_term.out_folder,
                                     "_".join(["inter_index", prefix]))
            tmp_sec = os.path.join(args_term.out_folder,
                                   "_".join(["inter_sec", prefix]))
            tran_file = os.path.join(self.tran_path,
                                     "_".join([prefix, "transcript.gff"]))
            gff_file = os.path.join(merge_path, prefix + ".gff")
            tmp_cand = tmp_cand = os.path.join(args_term.out_folder,
                                     "_".join(["term_candidates", prefix]))
            if os.path.exists(tran_file):
                print("Extracting seq of {0}".format(prefix))
                intergenic_seq(os.path.join(self.fasta_path, prefix + ".fa"),
                               tran_file, gff_file, tmp_seq, tmp_index, args_term)
                self._run_rnafold(args_term.RNAfold_path, tmp_seq, tmp_sec, prefix)
                extract_info_sec(tmp_sec, tmp_seq, tmp_index)
                os.remove(tmp_index)
                poly_t(tmp_seq, tmp_sec, gff_file, tran_file, tmp_cand, args_term)
            print("Detecting terminator for " + prefix)
            detect_coverage(
                tmp_cand, os.path.join(merge_path, prefix + ".gff"),
                os.path.join(self.tran_path, "_".join([
                    prefix, "transcript.gff"])),
                os.path.join(self.fasta_path, prefix + ".fa"),
                os.path.join(wig_path, "_".join([prefix, "forward.wig"])),
                os.path.join(wig_path, "_".join([prefix, "reverse.wig"])),
                os.path.join(self.tmps["hp_path"], "_".join([
                    prefix, self.tmps["hp_gff"]])), merge_wigs,
                os.path.join(self.outfolder["term"], "_".join([
                    prefix, self.suffixs["gff"]])),
                os.path.join(self.tmps["term_table"], "_".join([
                    prefix, "term_raw.csv"])), args_term)
        self.multiparser.combine_gff(args_term.gffs, self.outfolder["term"],
                                     None, "term")
        self._move_file(self.outfolder["term"], self.outfolder["csv"])

    def _remove_tmp_file(self, merge_wigs, args_term):
        self.helper.remove_tmp_dir(args_term.gffs)
        self.helper.remove_tmp_dir(args_term.fastas)
        if args_term.srnas is not None:
            self.helper.remove_tmp(args_term.srnas)
            shutil.rmtree(self.tmps["merge"])
        if (args_term.tex_wigs is not None) and (
                args_term.frag_wigs is not None):
            shutil.rmtree(merge_wigs)
        self.helper.remove_tmp_dir(args_term.trans)
        if "tmp_wig" in os.listdir(args_term.out_folder):
            shutil.rmtree(os.path.join(args_term.out_folder, "tmp_wig"))
        self.helper.remove_tmp(self.outfolder["term"])
        shutil.rmtree(self.tmps["transterm"])
        shutil.rmtree(self.tmps["term_table"])
        self.helper.remove_all_content(args_term.out_folder,
                                       "inter_seq_", "file")
        self.helper.remove_all_content(self.outfolder["term"],
                                       "_term.gff", "file")
        self.helper.remove_all_content(args_term.out_folder,
                                       "inter_sec_", "file")
        self.helper.remove_all_content(args_term.out_folder,
                                       "term_candidates_", "file")

    def _compute_stat(self, args_term):
        new_prefixs = []
        for gff in os.listdir(self.terms["all"]):
            if gff.endswith("_term_all.gff"):
                out_tmp = open(self.tmps["gff"], "w")
                out_tmp.write("##gff-version 3\n")
                new_prefix = gff.replace("_term_all.gff", "")
                new_prefixs.append(gff.replace("_term_all.gff", ""))
                num = 0
                fh = open(os.path.join(self.terms["all"], gff))
                for entry in self.gff_parser.entries(fh):
                    name = '%0*d' % (5, num)
                    entry.attributes["ID"] = (
                            entry.seq_id + "_terminator" + str(num))
                    entry.attributes["Name"] = "_".join(["terminator_" + name])
                    entry.attribute_string = ";".join([
                        "=".join(items) for items in entry.attributes.items()])
                    out_tmp.write("\t".join([entry.info_without_attributes,
                                  entry.attribute_string]) + "\n")
                    num += 1
                out_tmp.close()
                fh.close()
                shutil.move(self.tmps["gff"], os.path.join(self.terms["all"],
                            "_".join([new_prefix, self.suffixs["gff"]])))
        stat_path = os.path.join(args_term.out_folder, "statistics")
        for prefix in new_prefixs:
            stat_term(os.path.join(self.terms["all"],
                      "_".join([prefix, self.suffixs["gff"]])),
                      os.path.join(self.csvs["all"],
                      "_".join([prefix, self.suffixs["csv"]])),
                      os.path.join(stat_path,
                      "_".join(["stat", prefix + ".csv"])),
                      os.path.join(self.terms["best"],
                      "_".join([prefix, "term"])),
                      os.path.join(self.terms["express"],
                      "_".join([prefix, "term"])),
                      os.path.join(self.terms["non"],
                      "_".join([prefix, "term"])))
            shutil.move(os.path.join(self.terms["best"],
                        "_".join([prefix, self.suffixs["csv"]])),
                        os.path.join(self.csvs["best"],
                        "_".join([prefix, self.suffixs["csv"]])))
            shutil.move(os.path.join(self.terms["express"],
                        "_".join([prefix, self.suffixs["csv"]])),
                        os.path.join(self.csvs["express"],
                        "_".join([prefix, self.suffixs["csv"]])))
            shutil.move(os.path.join(self.terms["non"],
                        "_".join([prefix, self.suffixs["csv"]])),
                        os.path.join(self.csvs["non"],
                        "_".join([prefix, self.suffixs["csv"]])))
            os.remove(os.path.join(self.terms["all"],
                      "_".join([prefix, self.suffixs["allgff"]])))

    def _check_gff_file(self, folder):
        for file_ in os.listdir(folder):
            if file_.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(folder, file_))

    def _compare_term_tran(self, args_term, prefixs):
        '''searching the associated terminator to transcript'''
        self.multiparser.combine_gff(args_term.gffs, self.tran_path,
                                     None, "transcript")
        prefixs = []
        print("Comparing terminator with transcript now")
        for file_ in os.listdir(self.tran_path):
            if file_.endswith("_transcript.gff"):
                prefixs.append(file_.replace("_transcript.gff", ""))
        for type_ in ("best", "express", "all_candidates"):
            compare_term_tran(self.tran_path,
                              os.path.join(self.outfolder["term"], type_),
                              args_term.fuzzy_up_ta, args_term.fuzzy_down_ta,
                              args_term.out_folder, "terminator",
                              self.outfolder["term"], args_term.trans)
            for prefix in prefixs:
                shutil.move(
                    os.path.join(
                        args_term.out_folder, "statistics",
                        "stat_compare_transcript_terminator_" + prefix + ".csv"),
                    os.path.join(
                        args_term.out_folder, "statistics",
                        "_".join(["stat_compare_terminator_transcript", prefix,
                                  type_ + ".csv"])))

    def run_terminator(self, args_term):
        self._check_gff_file(args_term.gffs)
        self._check_gff_file(args_term.trans)
        self.multiparser.parser_fasta(args_term.fastas)
        if (not args_term.gffs) or (not args_term.fastas):
            print("Error: Please assign gff annotation folder "
                  "and fasta folder!!!")
            sys.exit()
        file_types, prefixs = self._convert_gff2rntptt(
                self.gff_path, self.fasta_path, args_term.srnas)
        self._combine_ptt_rnt(self.gff_path, file_types, self.srna_path)
        self._run_TransTermHP(args_term)
        self._convert_to_gff(prefixs, args_term)
        self.helper.remove_tmp(self.gff_path)
        self.multiparser.parser_gff(args_term.trans, "transcript")
        self.helper.check_make_folder(self.tmps["term_table"])
        self.multiparser.parser_gff(self.tmps["transterm"], self.tmps["hp"])
        merge_path = self._merge_sRNA(args_term.srnas, prefixs, self.gff_path)
        self._compute_intersection_forward_reverse(
                prefixs, merge_path, args_term.wig_path,
                args_term.merge_wigs, args_term)
        self._compute_stat(args_term)
        self._compare_term_tran(args_term, prefixs)
        self._remove_tmp_file(args_term.merge_wigs, args_term)



import os
import sys
from annogesiclib.helper import Helper
from annogesiclib.multiparser import Multiparser
from annogesiclib.optimize_TSSpredator import optimization


def get_length(fasta_file):
    length = 0
    with open(fasta_file) as fh:
        for line in fh:
            line = line.strip()
            if not line.startswith(">"):
                length = length + len(line)
    return length 


def optimize_tss(args_ops):
    if len(os.listdir(args_ops.gffs)) == 0:
        print("Error: There is no gff files!!!")
        sys.exit()
    if len(os.listdir(args_ops.fastas)) == 0:
        print("Error: There is no fasta files!!!")
        sys.exit()
    if len(os.listdir(args_ops.wigs)) == 0:
        print("Error: There is no wiggle files!!!")
        sys.exit()
    Multiparser().parser_wig(args_ops.wigs)
    Multiparser().parser_gff(args_ops.gffs, None)
    Multiparser().parser_fasta(args_ops.fastas)
    Multiparser().parser_gff(args_ops.manuals, None)
    gff_path = os.path.join(args_ops.gffs, "tmp")
    wig_path = os.path.join(args_ops.wigs, "tmp")
    fasta_path = os.path.join(args_ops.fastas, "tmp")
    manual_path = os.path.join(args_ops.manuals, "tmp")
    if "all" not in args_ops.strain_lengths.keys():
        for strain in args_ops.strain_lengths.keys():
            detect = False
            for man in os.listdir(manual_path):
                if strain == man.replace(".gff", ""):
                    detect = True
            if not detect:
                print("Error: There are strains in --strain_lengths "
                      "which is not contained in manual-detected "
                      "TSS gff files!")
                sys.exit()
    for man in os.listdir(manual_path):
        run = False
        prefix = man.replace(".gff", "")
        man_file = os.path.join(manual_path, man)
        if (prefix in args_ops.strain_lengths.keys()):
            length = args_ops.strain_lengths[prefix]
            run = True
        elif("all" in args_ops.strain_lengths.keys()):
            length = "all"
            run = True
        if run:
            for gff in os.listdir(gff_path):
                if (gff[:-4] == prefix) and (".gff" in gff):
                    gff_file = os.path.join(gff_path, gff)
                    break
            for fa in os.listdir(fasta_path):
                if (".".join(fa.split(".")[:-1]) == prefix) and (
                        ".fa" in fa):
                    fasta_file = os.path.join(fasta_path, fa)
                    break
            if length == "all":
                length = get_length(fasta_file)
            Helper().check_uni_attributes(gff_file)
            optimization(wig_path, fasta_file, gff_file, args_ops,
                         man_file, length, prefix)
            Helper().remove_all_content(os.path.join(
                args_ops.output_folder,
                "optimized_TSSpredator"), "config", "file")
            Helper().remove_all_content(os.path.join(
                args_ops.output_folder,
                "optimized_TSSpredator"), "Master", "dir")
    Helper().remove_tmp_dir(args_ops.wigs)
    Helper().remove_tmp_dir(args_ops.gffs)
    Helper().remove_tmp_dir(args_ops.fastas)
    Helper().remove_tmp_dir(args_ops.manuals)



import os
import sys
from annogesiclib.helper import Helper
from annogesiclib.detect_utr import detect_3utr, detect_5utr
from annogesiclib.multiparser import Multiparser


class UTRDetection(object):
    '''detection of UTR'''

    def __init__(self, args_utr):
        self.helper = Helper()
        self.multiparser = Multiparser()
        self.tss_path = os.path.join(args_utr.tsss, "tmp")
        self.tran_path = os.path.join(args_utr.trans, "tmp")
        self.utr5_path = os.path.join(args_utr.out_folder, "5UTR")
        self.utr3_path = os.path.join(args_utr.out_folder, "3UTR")
        self.utr5_stat_path = os.path.join(self.utr5_path, "statistics")
        self.utr3_stat_path = os.path.join(self.utr3_path, "statistics")

    def _check_folder(self, folder):
        if folder is None:
            print("Error: Lack required files!!!")
            sys.exit()

    def _check_gff(self, folder):
        for gff in os.listdir(folder):
            if gff.endswith(".gff"):
                self.helper.check_uni_attributes(os.path.join(folder, gff))

    def _compute_utr(self, args_utr):
        for gff in os.listdir(args_utr.gffs):
            if gff.endswith(".gff"):
                prefix = gff[:-4]
                tss = self.helper.get_correct_file(
                        self.tss_path, "_TSS.gff", prefix, None, None)
                tran = self.helper.get_correct_file(
                        self.tran_path, "_transcript.gff", prefix, None, None)
                if args_utr.terms:
                    term = self.helper.get_correct_file(
                                os.path.join(args_utr.terms, "tmp"),
                                "_term.gff", prefix, None, None)
                else:
                    term = None
                print("Computing 5'UTR of {0}".format(prefix))
                detect_5utr(tss, os.path.join(args_utr.gffs, gff),
                            tran, os.path.join(self.utr5_path, "gffs",
                            "_".join([prefix, "5UTR.gff"])), args_utr)
                print("Computing 3'UTR of {0}".format(prefix))
                detect_3utr(tran, os.path.join(args_utr.gffs, gff),
                            term, os.path.join(self.utr3_path, "gffs",
                            "_".join([prefix, "3UTR.gff"])), args_utr)
                self.helper.move_all_content(
                    os.getcwd(), self.utr5_stat_path, ["_5utr_length.png"])
                self.helper.move_all_content(
                    os.getcwd(), self.utr3_stat_path, ["_3utr_length.png"])

    def run_utr_detection(self, args_utr):
        self._check_folder(args_utr.tsss)
        self._check_folder(args_utr.gffs)
        self._check_folder(args_utr.trans)
        self._check_gff(args_utr.tsss)
        self._check_gff(args_utr.gffs)
        self._check_gff(args_utr.trans)
        self._check_gff(args_utr.terms)
        self.multiparser.parser_gff(args_utr.gffs, None)
        self.multiparser.parser_gff(args_utr.tsss, "TSS")
        self.multiparser.combine_gff(args_utr.gffs, self.tss_path, None, "TSS")
        self.multiparser.parser_gff(args_utr.trans, "transcript")
        self.multiparser.combine_gff(args_utr.gffs, self.tran_path,
                                     None, "transcript")
        if args_utr.terms:
            self.multiparser.parser_gff(args_utr.terms, "term")
            self.multiparser.combine_gff(args_utr.gffs,
                                         os.path.join(args_utr.terms, "tmp"),
                                         None, "term")
        self._compute_utr(args_utr)
        self.helper.remove_tmp_dir(args_utr.gffs)
        self.helper.remove_tmp_dir(args_utr.tsss)
        self.helper.remove_tmp_dir(args_utr.trans)
        self.helper.remove_tmp_dir(args_utr.terms)
        self.helper.remove_tmp(self.utr5_path)
        self.helper.remove_tmp(self.utr3_path)



import os
import csv
import math
import copy
import numpy as np
from annogesiclib.gff3 import Gff3Parser
from annogesiclib.coverage_detection import coverage_comparison
from annogesiclib.coverage_detection import replicate_comparison
from annogesiclib.lib_reader import read_wig, read_libs


def modify_attributes(pre_srna, srna, srna_type, input_type):
    if (srna_type == "UTR") or (srna_type == "both"):
        if pre_srna.attributes["sRNA_type"] != srna.attributes["sRNA_type"]:
            if input_type == "pre":
                if "antisense" in pre_srna.attributes["sRNA_type"]:
                    pre_srna.attributes["sRNA_type"] = (
                        srna.attributes["sRNA_type"])
            else:
                if "antisense" not in pre_srna.attributes["sRNA_type"]:
                    srna.attributes["sRNA_type"] = (
                        pre_srna.attributes["sRNA_type"])


def del_attributes(feature, entry):
    attributes = {}
    for key, value in entry.attributes.items():
        if feature not in key:
            attributes[key] = value
    return attributes


def detect_overlap(srna, pre_srna, srna_type, overlap):
    '''check the sRNA is overlap with other sRNA or not'''
    if (srna.seq_id == pre_srna.seq_id) and (
            srna.strand == pre_srna.strand):
        if (pre_srna.start >= srna.start) and (
                pre_srna.end <= srna.end):
            modify_attributes(pre_srna, srna, srna_type, None)
            overlap = True
        elif (pre_srna.start >= srna.start) and (
                pre_srna.start <= srna.end) and (
                pre_srna.end >= srna.end):
            modify_attributes(pre_srna, srna, srna_type, None)
            overlap = True
        elif (pre_srna.start <= srna.start) and (
                pre_srna.end >= srna.start) and (
                pre_srna.end <= srna.end):
            modify_attributes(pre_srna, srna, srna_type, None)
            overlap = True
        elif (pre_srna.start <= srna.start) and (
                pre_srna.end >= srna.end):
            overlap = True
            modify_attributes(pre_srna, srna, srna_type, "pre")
    return overlap


def merge_tss_pro(pre_srna, srna, feature):
    if (feature not in pre_srna.attributes.keys()) and (
            feature in srna.attributes.keys()):
        if srna.attributes[feature] != "NA":
            pre_srna.attributes[feature] = srna.attributes[feature]
    elif (feature in pre_srna.attributes.keys()) and (
            feature in srna.attributes.keys()):
        if (pre_srna.attributes[feature] == "NA") and (
                srna.attributes[feature] != "NA"):
            pre_srna.attributes[feature] = srna.attributes[feature]
        elif (srna.attributes[feature] not in
              pre_srna.attributes[feature]) and (
                srna.attributes[feature] != "NA"):
            pre_srna.attributes[feature] = ",".join(
                                              [pre_srna.attributes[feature],
                                               srna.attributes[feature]])


def modify_overlap(pre_srna, srna):
    '''If the sRNA is overlap with other sRNA, 
    it will modify the position and attributes of gff file'''
    merge_tss_pro(pre_srna, srna, "with_TSS")
    merge_tss_pro(pre_srna, srna, "end_cleavage")
    if (srna.attributes["sRNA_type"] == "5utr") or (
            srna.attributes["sRNA_type"] == "3utr") or (
            srna.attributes["sRNA_type"] == "interCDS"):
        merge_tss_pro(pre_srna, srna, "start_cleavage")
    if (srna.start < pre_srna.start):
        pre_srna.start = srna.start
    if (srna.end > pre_srna.end):
        pre_srna.end = srna.end
    return pre_srna


def merge_srna(srnas, srna_type):
    '''Merge the overlaped sRNA'''
    final_srnas = []
    first = True
    pre_srna = ""
    for srna in srnas:
        if srna.feature != "ncRNA":
            srna.feature = "ncRNA"
        if "with_TSS" in srna.attributes.keys():
            if srna.attributes["with_TSS"] == "False":
                srna.attributes["with_TSS"] = "NA"
        else:
            srna.attributes["with_TSS"] = "NA"
        if "end_cleavage" in srna.attributes.keys():
            if srna.attributes["end_cleavage"] == "False":
                srna.attributes["end_cleavage"] = "NA"
        else:
            srna.attributes["end_cleavage"] = "NA"
        overlap = False
        if first:
            first = False
            pre_srna = srna
        else:
            if (srna.seq_id != pre_srna.seq_id):
                if not overlap:
                    if pre_srna not in final_srnas:
                        final_srnas.append(pre_srna)
                pre_srna = srna
                continue
            overlap = detect_overlap(srna, pre_srna, srna_type, overlap)
            if overlap:
                pre_srna = modify_overlap(pre_srna, srna)
                if (srna.attributes["sRNA_type"] != "antisense") and (
                        pre_srna.attributes["sRNA_type"] == "antisense"):
                    pre_srna = srna
            else:
                if pre_srna not in final_srnas:
                    final_srnas.append(pre_srna)
                pre_srna = srna
        srna.source = "ANNOgesic"
    if overlap:
        pre_srna = modify_overlap(pre_srna, srna)
        if pre_srna not in final_srnas:
            final_srnas.append(pre_srna)
    else:
        if srna not in final_srnas:
            final_srnas.append(srna)
    return final_srnas


def read_gff(gff_file, type_):
    datas = []
    if os.path.exists(gff_file):
        for entry in Gff3Parser().entries(open(gff_file)):
            if type_ == "sRNA":
                datas.append(entry)
            elif type_ == "tss":
                datas.append(entry)
            else:
                if (entry.feature == "CDS") or (
                        entry.feature == "tRNA") or (
                        entry.feature == "rRNA"):
                    datas.append(entry)
        datas = sorted(datas, key=lambda k: (k.seq_id, k.start,
                                             k.end, k.strand))
    return datas


def read_table(table_file, file_type):
    datas = []
    if os.path.exists(table_file):
        f_h = open(table_file, "r")
        for row in csv.reader(f_h, delimiter='\t'):
            datas.append(import_data(row, file_type))
    return datas


def merge_incds_utr(utrs, inters):
    '''merge the sRNA within CDS and UTR-derived sRNA'''
    new_inters = []
    for inter in inters:
        remove = False
        for utr in utrs:
            if inter.source == "in_CDS":
                if (inter.seq_id == utr.seq_id) and (
                        inter.strand == utr.strand):
                    if ((inter.end < utr.end) and (
                             inter.end > utr.start) and (
                             inter.start <= utr.start)) or (
                            (inter.start > utr.start) and (
                             inter.start < utr.end) and (
                             inter.end >= utr.end)) or (
                            (inter.end >= utr.end) and (
                             inter.start <= utr.start)) or (
                            (inter.end <= utr.end) and (
                             inter.start >= utr.start)):
                        utr.start = min(inter.start, utr.start)
                        utr.end = max(inter.end, utr.end)
                        remove = True
        if not remove:
            new_inters.append(inter)
    return new_inters


def compare_srna_cds(srna, cdss, cutoff_overlap):
    '''compare sRNA and CDS to get the information of 
    overlap between sRNA and CDS'''
    detect = False
    overlap = False
    for cds in cdss:
        if (srna.seq_id == cds.seq_id) and (
                srna.strand == cds.strand):
            if ((srna.end < cds.end) and (
                     srna.end > cds.start) and (
                     srna.start <= cds.start)) or (
                    (srna.start > cds.start) and (
                     srna.start < cds.end) and (
                     srna.end >= cds.end)) or (
                    (srna.end >= cds.end) and (
                     srna.start <= cds.start)) or (
                    (srna.end <= cds.end) and (
                     srna.start >= cds.start)):
                overlap = True
                per_c = float(min(srna.end, cds.end) - max(
                        srna.start, cds.start)) / float(cds.end - cds.start)
                if per_c <= cutoff_overlap:
                    if "product" in cds.attributes.keys():
                        cds_name = "".join([
                            cds.feature, ":", str(cds.start),
                            "-", str(cds.end), "_", cds.strand,
                            "(", cds.attributes["product"], ")"])
                    else:
                        cds_name = "".join([
                            cds.feature, ":", str(cds.start),
                            "-", str(cds.end), "_", cds.strand])
                    if "overlap_cds" not in srna.attributes.keys():
                        srna.attributes["overlap_cds"] = cds_name
                        srna.attributes["overlap_percent"] = str(per_c)
                    else:
                        srna.attributes["overlap_cds"] = (
                            ",".join([srna.attributes["overlap_cds"],
                                      cds_name]))
                        srna.attributes["overlap_percent"] = (
                            ",".join([srna.attributes["overlap_percent"],
                                      str(per_c)]))
                    detect = True
    if not overlap:
        srna.attributes["overlap_cds"] = "NA"
        srna.attributes["overlap_percent"] = "NA"
        return srna
    elif overlap and detect:
        return srna
    else:
        return None


def merge_srna_gff(gffs, in_cds, cutoff_overlap, gff_file):
    '''merge all types of sRNA and print to one gff files'''
    out = open(gffs["merge"], "w")
    out.write("##gff-version 3\n")
    utrs = read_gff(gffs["utr"], "sRNA")
    inters = read_gff(gffs["normal"], "sRNA")
    cdss = read_gff(gff_file, "CDS")
    num_srna = 0
    srnas = None
    if (in_cds) and (len(utrs) != 0) and (len(inters) != 0):
        inters = merge_incds_utr(utrs, inters)
    if (len(utrs) != 0) and (len(inters) != 0):
        pre_srnas = inters + utrs
        pre_srnas = sorted(pre_srnas, key=lambda x: (
                           x.seq_id, x.start, x.end, x.strand))
        srnas = merge_srna(pre_srnas, "both")
    elif len(utrs) != 0:
        srnas = merge_srna(utrs, "UTR")
    elif len(inters) != 0:
        srnas = merge_srna(inters, "inter")
    sort_srnas = sorted(srnas, key=lambda x: (x.seq_id, x.start,
                                              x.end, x.strand))
    for srna in sort_srnas:
        new_srna = compare_srna_cds(srna, cdss, cutoff_overlap)
        if new_srna:
            new_srna.attributes["ID"] = (
                    new_srna.seq_id + "_srna" + str(num_srna))
            name = '%0*d' % (5, num_srna)
            new_srna.attributes["Name"] = "sRNA_" + str(name)
            new_srna.attributes = del_attributes("best_high_coverage",
                                                 new_srna)
            new_srna.attributes = del_attributes("best_low_coverage",
                                                 new_srna)
            new_srna.attributes = del_attributes("best_avg_coverage",
                                                 new_srna)
            attribute_string = ";".join([
                "=".join(items) for items in new_srna.attributes.items()])
            new_srna.info_without_attributes = (
                "\t".join([str(field) for field in [
                    new_srna.seq_id, new_srna.source, new_srna.feature,
                    new_srna.start, new_srna.end, new_srna.score,
                    new_srna.strand, new_srna.phase]]))
            out.write(new_srna.info_without_attributes + "\t" +
                      attribute_string + "\n")
            num_srna += 1
    out.close()


def import_data(row, type_):
    if type_ == "inter":
        return {"strain": row[0], "name": row[1],
                "start": int(row[2]), "end": int(row[3]),
                "strand": row[4], "libs": row[5],
                "detect": row[6], "avg": row[7],
                "high": row[8], "low": row[9],
                "detail": row[11], "tss": row[10]}
    if type_ == "utrr":
        return {"strain": row[0], "name": row[1],
                "start": int(row[2]), "end": int(row[3]),
                "strand": row[4], "libs": row[5],
                "detect": row[6], "avg": row[7],
                "high": row[8], "low": row[9],
                "detail": row[10]}


def check_real_cut(inter_cuts, tss_type, cut):
    for tss, value in inter_cuts.items():
        if tss in tss_type.lower():
            if cut is None:
                cut = inter_cuts[tss]
            else:
                if cut > inter_cuts[tss]:
                    cut = inter_cuts[tss]
    if cut is None:
        if "no_tss" not in inter_cuts.keys():
            cut = 0
        else:
            cut = inter_cuts["no_tss"]
    return cut


def get_cutoff(srna, tsss, type_, tables, args_srna):
    if type_ == "inter":
        tss_type = None
        inter_cuts = {"frag": {}, "tex": {}, "notex": {}}
        fh = open(os.path.join(args_srna.out_folder, "tmp_cutoff_inter"), "r")
        for row in csv.reader(fh, delimiter='\t'):
            inter_cuts[row[0]][row[1]] = float(row[2])
        if tsss is not None:
            for tss in tsss:
                if (srna.seq_id == tss.seq_id) and (
                        srna.strand == tss.strand):
                    if srna.strand == "+":
                        if math.fabs(srna.start -
                           tss.start) <= args_srna.fuzzy_inter:
                            tss_type = tss.attributes["type"]
                            if srna.start == tss.start:
                                break
                    else:
                        if (math.fabs(srna.end - tss.start) <=
                                args_srna.fuzzy_inter):
                            tss_type = tss.attributes["type"]
                            if srna.end == tss.start:
                                break
        cut = {"frag": None, "tex": None, "notex": None}
        if tss_type is None:
            tss_type = "no_tss"
        for key, types in inter_cuts.items():
            cut[key] = check_real_cut(types, tss_type, cut[key])
    elif type_ == "utr":
        cut = {}
        fh = open(os.path.join(args_srna.out_folder, "tmp_median"), "r")
        for row in csv.reader(fh, delimiter='\t'):
            if (row[0] == srna.seq_id) and (
                    row[1] == srna.attributes["sRNA_type"]):
                if row[1] not in cut.keys():
                    cut[row[1]] = {}
                cut[row[1]][row[2]] = {"median": float(row[3])}
    fh.close()
    return cut


def devide_covers(covers):
    frag_covers = {}
    tex_covers = {}
    for cond, tracks in covers.items():
        if "frag" in cond:
            frag_covers[cond] = tracks
        elif "tex" in cond:
            tex_covers[cond] = tracks
    return frag_covers, tex_covers


def merge_srna_datas(srna_datas_tex, srna_datas_frag):
    if (len(srna_datas_tex["conds"]) != 0) and (
            len(srna_datas_frag["conds"]) != 0):
        srna_datas = copy.deepcopy(srna_datas_tex)
        for key, values in srna_datas_frag.items():
            if key == "conds":
                srna_datas["conds"] = dict(srna_datas["conds"], **values)
            elif key == "best":
                if srna_datas["best"] < values:
                    srna_datas["best"] = values
                    srna_datas["high"] = srna_datas_frag["high"]
                    srna_datas["low"] = srna_datas_frag["low"]
                    srna_datas["track"] = srna_datas_frag["track"]
            elif key == "detail":
                srna_datas["detail"] = srna_datas["detail"] + values
    elif len(srna_datas_tex["conds"]) != 0:
        srna_datas = copy.deepcopy(srna_datas_tex)
    elif len(srna_datas_frag["conds"]) != 0:
        srna_datas = copy.deepcopy(srna_datas_frag)
    else:
        srna_datas = copy.deepcopy(srna_datas_tex)
    return srna_datas


def compare_table(srna, tables, type_, wigs_f, wigs_r, texs,
                  out, tsss, args_srna):
    detect = False
    tss_pro, end_pro = get_tss_pro(type_, srna)
    if not detect:
        if type_ == "inter":
            if srna.strand == "+":
                covers = get_coverage(wigs_f, srna)
            else:
                covers = get_coverage(wigs_r, srna)
            cut = get_cutoff(srna, tsss, type_, tables, args_srna)
            frag_covers, tex_covers = devide_covers(covers)
            srna_datas_tex = replicate_comparison(
                args_srna, tex_covers, srna.strand, "normal", None, None,
                None, cut["notex"], cut["tex"], texs)
            srna_datas_frag = replicate_comparison(
                args_srna, frag_covers, srna.strand, "normal", None, None,
                None, None, cut["frag"], texs)
            srna_datas = merge_srna_datas(srna_datas_tex, srna_datas_frag)
        elif type_ == "utr":
            if srna.strand == "+":
                covers = get_coverage(wigs_f, srna)
            else:
                covers = get_coverage(wigs_r, srna)
            cut = get_cutoff(srna, tsss, type_, tables, args_srna)
            srna_datas = replicate_comparison(
                args_srna, covers, srna.strand, "sRNA_utr_derived",
                cut[srna.attributes["sRNA_type"]],
                cut, srna.attributes["sRNA_type"], None, cut, texs)
        if len(srna_datas["conds"]) != 0:
            out.write("{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}"
                      "\t{8}\t{9}\t{10}\t{11}\t".format(
                          srna.seq_id, srna.attributes["Name"], srna.start,
                          srna.end, srna.strand,
                          ";".join(srna_datas["conds"].keys()),
                          ";".join(srna_datas["conds"].values()), tss_pro,
                          end_pro, srna_datas["best"], srna_datas["high"],
                          srna_datas["low"]))
            if not args_srna.table_best:
                first = True
                for data in srna_datas["detail"]:
                    if first:
                        out.write("{0}(avg={1};high={2};low={3})".format(
                                  data["track"], data["avg"], data["high"],
                                  data["low"]))
                        first = False
                    else:
                        out.write(";{0}(avg={1};high={2};low={3})".format(
                                  data["track"], data["avg"], data["high"],
                                  data["low"]))
            else:
                out.write("{0}(avg={1};high={2};low={3})".format(
                          srna_datas["track"], srna_datas["best"],
                          srna_datas["high"], srna_datas["low"]))
            out.write("\t{0}\t{1}\n".format(
                      srna.attributes["overlap_cds"].replace(",", ";"),
                      srna.attributes["overlap_percent"].replace(",", ";")))


def check_start_and_end(start, end, covers):
    if (start - 2) < 0:
        c_start = 0
    else:
        c_start = start - 2
    if (end + 2) > len(covers):
        c_end = len(covers)
    else:
        c_end = end + 2
    return c_start, c_end


def get_coverage(wigs, srna):
    cover_sets = {"high": -1, "low": -1, "total": 0, "diff": 0}
    poss = {"high": 0, "low": 0, "pos": 0}
    srna_covers = {}
    for wig_strain, conds in wigs.items():
        if wig_strain == srna.seq_id:
            for cond, tracks in conds.items():
                srna_covers[cond] = []
                for lib_name, covers in tracks.items():
                    track = lib_name.split("|")[-3]
                    lib_strand = lib_name.split("|")[-2]
                    lib_type = lib_name.split("|")[-1]
                    cover_sets["total"] = 0
                    cover_sets["diff"] = 0
                    first = True
                    c_start, c_end = check_start_and_end(
                            srna.start, srna.end, covers)
                    covers = covers[c_start: c_end]
                    if srna.strand == "-":
                        covers = covers[::-1]
                    pos = 0
                    for cover in covers:
                        if (lib_strand == srna.strand):
                            if srna.strand == "+":
                                cover_pos = c_start + pos
                            else:
                                cover_pos = c_end - pos
                            if (srna.start <= cover_pos) and (
                                    srna.end >= cover_pos):
                                cover_sets["total"] = (cover_sets["total"] +
                                                       cover)
                                first = coverage_comparison(
                                        cover, cover_sets, poss,
                                        first, srna.strand, cover_pos)
                            else:
                                if (srna.strand == "+") and (
                                        cover_pos > srna.end):
                                    cover_sets_pos = cover_pos
                                    break
                                elif (srna.strand == "-") and (
                                        cover_pos < srna.start):
                                    cover_sets["pos"] = cover_pos
                                    break
                        pos += 1
                    avg = cover_sets["total"] / float(
                            srna.end - srna.start + 1)
                    srna_covers[cond].append({"track": track,
                                              "high": cover_sets["high"],
                                              "low": cover_sets["low"],
                                              "avg": avg,
                                              "pos": poss["pos"],
                                              "type": lib_type,
                                              "final_start": srna.start,
                                              "final_end": srna.end})
    return srna_covers


def get_tss_pro(type_, srna):
    '''check the sRNA is associated with TSS or processing site'''
    if type_ == "utr":
        if (srna.attributes["with_TSS"] != "NA") and (
                srna.attributes["start_cleavage"] != "NA"):
            tss_pro = ";".join([srna.attributes["with_TSS"],
                                srna.attributes["start_cleavage"]])
        elif (srna.attributes["with_TSS"] != "NA"):
            tss_pro = srna.attributes["with_TSS"]
        elif srna.attributes["start_cleavage"] != "NA":
            tss_pro = srna.attributes["start_cleavage"]
        else:
            tss_pro = "NA"
        if (srna.attributes["end_cleavage"] != "NA"):
            end_pro = srna.attributes["end_cleavage"]
        else:
            end_pro = "NA"
        tss_pro = tss_pro.replace(",", ";")
        end_pro = end_pro.replace(",", ";")
    elif type_ == "inter":
        tss_pro = ""
        end_pro = ""
        if (srna.attributes["with_TSS"] != "NA"):
            tss_pro = srna.attributes["with_TSS"].replace(",", ";")
        else:
            tss_pro = "NA"
        if (srna.attributes["end_cleavage"] != "NA"):
            end_pro = srna.attributes["end_cleavage"].replace(",", ";")
        else:
            end_pro = "NA"
    return tss_pro, end_pro


def free_memory(paras):
    for data in paras:
        del(data)


def merge_srna_table(srna_file, csvs, wigs_f, wigs_r,
                     tss_file, args_srna):
    libs, texs = read_libs(args_srna.libs, args_srna.merge_wigs)
#    wigs_f = read_wig(wig_f_file, "+", libs)
#    wigs_r = read_wig(wig_r_file, "-", libs)
    srnas = read_gff(srna_file, "sRNA")
    if tss_file is not None:
        tsss = read_gff(tss_file, "tss")
    else:
        tsss = None
    inters = read_table(csvs["normal"], "inter")
    utrs = read_table(csvs["utr"], "utr")
    out = open(csvs["merge"], "w")
    for srna in srnas:
        if ("5utr" in srna.attributes["sRNA_type"]) or (
                "3utr" in srna.attributes["sRNA_type"]) or (
                "interCDS" in srna.attributes["sRNA_type"]):
            compare_table(srna, utrs, "utr", wigs_f, wigs_r,
                          texs, out, tsss, args_srna)
        elif ("intergenic" in srna.attributes["sRNA_type"]) or (
                "in_CDS" in srna.attributes["sRNA_type"]) or (
                "antisense" in srna.attributes["sRNA_type"]):
            compare_table(srna, inters, "inter", wigs_f, wigs_r,
                          texs, out, tsss, args_srna)
    out.close()
    paras = [wigs_r, wigs_f, srnas, tsss, inters, utrs]
    free_memory(paras)



import os
import sys
import csv
import shutil
from annogesiclib.seq_editer import SeqEditer
from annogesiclib.helper import Helper


class Multiparser(object):

    def __init__(self):
        self.seq_editer = SeqEditer()
        self.helper = Helper()
        self.tmp_fa = "tmp.fa"
        self.tmp_gff = "tmp.gff"
        self.tmp_wig_forward = "tmp_forward.wig"
        self.tmp_wig_reverse = "tmp_reverse.wig"

    def combine_fasta(self, ref_folder, tar_folder, ref_feature):
        '''combine multiple fasta files'''
        tar_merge = os.path.join(tar_folder, "merge_tmp")
        change = False
        if ref_feature is None:
            ref_feature = ""
        else:
            ref_feature = "_" + ref_feature
        self.helper.check_make_folder(tar_merge)
        for folder in os.listdir(ref_folder):
            files = []
            if "_folder" in folder:
                datas = folder.split("_folder")
                if ref_feature == "":
                    prefix = datas[0][:-4]
                elif ref_feature == "_fasta":
                    if datas[0].endswith(".fa"):
                        prefix = datas[0][:-3]
                    elif datas[0].endswith(".fna"):
                        prefix = datas[0][:-4]
                    elif datas[0].endswith(".fasta"):
                        prefix = datas[0][:-6]
                else:
                    datas = datas[0][:-4]
                    datas = datas.split(ref_feature)
                    prefix = datas[0]
                print("Merging fasta file of " + prefix)
                for file_ in os.listdir("/".join([ref_folder, folder])):
                    if ref_feature == "":
                        files.append(file_[:-4])
                    elif ref_feature == "_fasta":
                        files.append(file_[:-3])
                    else:
                        filename = file_.split(ref_feature)
                        files.append(filename[0])
                for tar in os.listdir(tar_folder):
                    if tar.endswith(".fa") or \
                       tar.endswith(".fna") or \
                       tar.endswith(".fasta"):
                        filename = ".".join((tar.split("."))[:-1])
                        for file_ in files:
                            if filename == file_:
                                self.helper.merge_file(
                                     os.path.join(tar_folder, tar),
                                     os.path.join(tar_folder, self.tmp_fa))
                                change = True
                if change:
                    change = False
                    shutil.move(os.path.join(tar_folder, self.tmp_fa),
                                os.path.join(tar_merge, prefix + ".fa"))
        self.helper.remove_all_content(tar_folder, ".fa", "file")
        self.helper.move_all_content(tar_merge, tar_folder, None)
        shutil.rmtree(tar_merge)

    def get_prefix(self, folder, ref_feature):
        datas = folder.split("_folder")
        if ref_feature == "":
            prefix = datas[0][:-4]
        elif ref_feature == "_fasta":
            if datas[0].endswith(".fa"):
                prefix = datas[0][:-3]
            elif datas[0].endswith(".fna"):
                prefix = datas[0][:-4]
            elif datas[0].endswith(".fasta"):
                prefix = datas[0][:-6]
        else:
            datas = datas[0][:-4]
            datas = datas.split(ref_feature)
            prefix = datas[0]
        return prefix

    def combine_wig(self, ref_folder, tar_folder, ref_feature, libs):
        '''combine multiple wig files'''
        tar_merge = os.path.join(tar_folder, "merge_tmp")
        change_f = False
        change_r = False
        if ref_feature is None:
            ref_feature = ""
        else:
            ref_feature = "_" + ref_feature
        self.helper.check_make_folder(tar_merge)
        for folder in os.listdir(ref_folder):
            files = []
            if "_folder" in folder:
                prefix = self.get_prefix(folder, ref_feature)
                print("Merging wig file of " + prefix)
                for file_ in os.listdir(os.path.join(ref_folder, folder)):
                    if ref_feature == "":
                        files.append(file_[:-4])
                    elif ref_feature == "_fasta":
                        files.append(file_[:-3])
                    else:
                        filename = file_.split(ref_feature)
                        files.append(filename[0])
                for tar in os.listdir(tar_folder):
                    filename = tar.split("_STRAIN_")
                    for file_ in files:
                        if (tar.endswith(".wig")) and (
                                file_ == filename[-1][:-4]):
                            for lib in libs:
                                if (filename[0] in lib) and (lib[-1] == "+"):
                                    self.helper.merge_file(
                                        os.path.join(tar_folder, tar),
                                        os.path.join(tar_folder,
                                                     self.tmp_wig_forward))
                                    change_f = True
                                elif (filename[0] in lib) and (lib[-1] == "-"):
                                    self.helper.merge_file(
                                        os.path.join(tar_folder, tar),
                                        os.path.join(tar_folder,
                                                     self.tmp_wig_reverse))
                                    change_r = True
                if change_f and change_r:
                    change_f = False
                    change_r = False
                    shutil.move(os.path.join(tar_folder, self.tmp_wig_forward),
                                os.path.join(tar_merge,
                                             prefix + "_forward.wig"))
                    shutil.move(os.path.join(tar_folder, self.tmp_wig_reverse),
                                os.path.join(tar_merge,
                                             prefix + "_reverse.wig"))
                else:
                    print("Error: .wig files should be compose of "
                          "forward or reverse files.")
                    sys.exit()
        self.helper.remove_all_content(tar_folder, ".wig", "file")
        self.helper.move_all_content(tar_merge, tar_folder, None)
        shutil.rmtree(tar_merge)

    def combine_gff(self, ref_folder, tar_folder, ref_feature, tar_feature):
        '''combine multiple gff files'''
        tar_merge = os.path.join(tar_folder, "merge_tmp")
        change = False
        if tar_feature is None:
            tar_feature = ""
        else:
            tar_feature = "_" + tar_feature
        if ref_feature is None:
            ref_feature = ""
        else:
            ref_feature = "_" + ref_feature
        self.helper.check_make_folder(tar_merge)
        for folder in os.listdir(ref_folder):
            files = []
            if "_folder" in folder:
                datas = folder.split("_folder")
                if ref_feature == "":
                    prefix = datas[0][:-4]
                elif ref_feature == "_fasta":
                    if datas[0].endswith(".fa"):
                        prefix = datas[0][:-3]
                    elif datas[0].endswith(".fna"):
                        prefix = datas[0][:-4]
                    elif datas[0].endswith(".fasta"):
                        prefix = datas[0][:-6]
                else:
                    datas = datas[0][:-4]
                    datas = datas.split(ref_feature)
                    prefix = datas[0]
                print("Merging gff file of " + prefix + tar_feature)
                for file_ in os.listdir(os.path.join(ref_folder, folder)):
                    if ref_feature == "":
                        files.append(file_[:-4])
                    elif ref_feature == "_fasta":
                        files.append(file_[:-3])
                    else:
                        filename = file_.split(ref_feature)
                        files.append(filename[0])
                for tar in os.listdir(tar_folder):
                    for file_ in files:
                        if (".gff" in tar) and (
                                file_ + tar_feature == tar[:-4]):
                            self.helper.merge_file(
                                 os.path.join(tar_folder, tar),
                                 os.path.join(tar_folder, self.tmp_gff))
                            change = True
                if change:
                    change = False
                    shutil.move(os.path.join(tar_folder, self.tmp_gff),
                                os.path.join(tar_folder, "merge_tmp",
                                prefix + tar_feature + ".gff"))
        self.helper.remove_all_content(tar_folder, ".gff", "file")
        self.helper.move_all_content(tar_merge, tar_folder, None)
        shutil.rmtree(tar_merge)

    def parser_fasta(self, fastas):
        '''parser the fasta file based on strain'''
        par_tmp = os.path.join(fastas, "tmp")
        first = True
        out = None
        out_t = None
        detect = False
        for fasta in os.listdir(fastas):
            if (fasta.endswith(".fasta") or
                    fasta.endswith(".fa") or
                    fasta.endswith(".fna")):
                detect = True
                self.seq_editer.modify_header(os.path.join(fastas, fasta))
        self.helper.check_make_folder(par_tmp)
        if not detect:
            print("Error: there are folders which conatin no fasta files! "
                  "The files should end with .fa or .fna or .fasta!")
            sys.exit()
        for fasta in os.listdir(fastas):
            if ("_folder" not in fasta) and ("tmp" != fasta):
                if (fasta.endswith(".fa")) or \
                   (fasta.endswith(".fna")) or \
                   (fasta.endswith(".fasta")):
                    out_path = os.path.join(fastas, fasta + "_folder")
                    print("Parsing " + fasta)
                    self.helper.check_make_folder(out_path)
                    with open(os.path.join(fastas, fasta), "r") as f_f:
                        for line in f_f:
                            if line[0] == ">":
                                line = line.strip()
                                if ("|" in line) and (
                                        len(line.split("|")) > 4):
                                    strain = line.split("|")
                                    name = strain[3]
                                else:
                                    name = line[1:]
                                if first:
                                    first = False
                                else:
                                    out.close()
                                    out_t.close()
                                out = open(os.path.join(
                                           out_path, name + ".fa"), "w")
                                out_t = open(os.path.join(
                                             par_tmp, name + ".fa"), "w")
                                out.write(">" + name + "\n")
                                out_t.write(">" + name + "\n")
                            else:
                                out.write(line)
                                out_t.write(line)
        out.close()
        out_t.close()

    def parser_gff(self, gff_folder, feature):
        '''parser gff file based on strain'''
        par_tmp = os.path.join(gff_folder, "tmp")
        out = None
        out_t = None
        first = True
        detect = False
        if feature is None:
            feature = ""
        else:
            feature = "_" + feature
        self.helper.check_make_folder(par_tmp)
        for filename in os.listdir(gff_folder):
            pre_seq_id = ""
            if ("_folder" not in filename) and ("tmp" != filename):
                out_path = os.path.join(gff_folder, filename + "_folder")
                if ".gff" in filename:
                    detect = True
                    print("Parsing " + filename)
                    self.helper.check_make_folder(out_path)
                    self.helper.sort_gff(os.path.join(gff_folder, filename),
                                         os.path.join(gff_folder, "tmp.gff"))
                    f_h = open(os.path.join(gff_folder, "tmp.gff"), "r")
                    for row in csv.reader(f_h, delimiter="\t"):
                        if row[0].startswith("#"):
                            continue
                        else:
                            if pre_seq_id == row[0]:
                                out.write("\t".join(row) + "\n")
                                out_t.write("\t".join(row) + "\n")
                            else:
                                if first:
                                    first = False
                                else:
                                    out.close()
                                    out_t.close()
                                out = open(os.path.join(out_path,
                                           row[0] + feature + ".gff"), "w")
                                out_t = open(os.path.join(par_tmp,
                                             row[0] + feature + ".gff"), "w")
                                pre_seq_id = row[0]
                                out.write("\t".join(row) + "\n")
                                out_t.write("\t".join(row) + "\n")
                    f_h.close()
        if not detect:
            print("Error: There are folders which contain no gff3 files! "
                  "The files should end with .gff!")
            sys.exit()
        if os.path.exists(os.path.join(gff_folder, "tmp.gff")):
            os.remove(os.path.join(gff_folder, "tmp.gff"))
        out.close()
        out_t.close()

    def parser_wig(self, wig_folder):
        '''parser the wig file based on strain'''
        par_tmp = os.path.join(wig_folder, "tmp")
        first = True
        out = None
        out_t = None
        detect = False
        self.helper.check_make_folder(par_tmp)
        for filename in os.listdir(wig_folder):
            track_info = ""
            if ("_folder" not in filename) and ("tmp" != filename):
                out_path = os.path.join(wig_folder, filename + "_folder")
                if ".wig" in filename:
                    detect = True
                    print("Parsing {0}".format(filename))
                    self.helper.check_make_folder(out_path)
                    with open(os.path.join(wig_folder, filename), "r") as w_f:
                        for line in w_f:
                            line = line.split(" ")
                            if (line[0] == "track"):
                                track_info = " ".join(line)
                            if (line[0] == "variableStep"):
                                strain = line[1].split("=")
                                if first:
                                    first = False
                                else:
                                    out.close()
                                    out_t.close()
                                out = open("".join([
                                    os.path.join(out_path, filename[:-4]),
                                    "_STRAIN_", strain[1], ".wig"]), "w")
                                out_t = open("".join([
                                    os.path.join(wig_folder, "tmp",
                                                 filename[:-4]),
                                    "_STRAIN_", strain[1], ".wig"]), "w")
                                if track_info != "":
                                    out.write(track_info)
                                    out_t.write(track_info)
                                out.write(" ".join(line))
                                out_t.write(" ".join(line))
                            if (line[0] != "track") and (
                                    line[0] != "variableStep"):
                                out.write(" ".join(line))
                                out_t.write(" ".join(line))
        if not detect:
            print("Error: There are folders which contain no wig files! "
                  "The files should end with .wig!")
            sys.exit()
        out.close()
        out_t.close()



import os
import sys
import csv
import shutil
from glob import glob
from subprocess import call
from annogesiclib.multiparser import Multiparser
from annogesiclib.seq_editer import SeqEditer
from annogesiclib.transcript_SNP import snp_detect
from annogesiclib.helper import Helper


class SNPCalling(object):
    '''detection of SNP'''

    def __init__(self, args_snp):
        self.multiparser = Multiparser()
        self.seq_editer = SeqEditer()
        self.helper = Helper()
        if args_snp.types == "closed_strain":
            file_type = "compare_closed_and_updated_references"
        else:
            file_type = "mutations_of_query_strain"
        self.seq_path = os.path.join(args_snp.out_folder, file_type, "seqs")
        self.stat_path = os.path.join(args_snp.out_folder, file_type,
                                      "statistics")
        self.fig_path = os.path.join(self.stat_path, "figs")
        self.helper.check_make_folder(self.fig_path)
        self.outputs = {"table": os.path.join(
                        args_snp.out_folder, file_type, "SNP_table"),
                        "raw": os.path.join(
                        args_snp.out_folder, file_type, "SNP_raw_outputs"),
                        "tmp": os.path.join(args_snp.out_folder, "tmp_bcf"),
                        "depth": os.path.join(args_snp.out_folder, "tmp_depth")}
        self.bams = {"whole": os.path.join(args_snp.out_folder,
                                           "whole_reads.bam"),
                     "sort": os.path.join(args_snp.out_folder,
                                          "whole_reads_sorted.bam"),
                     "bams": []}
        self.header = os.path.join(args_snp.out_folder, "header")
        self.baqs = {"with": "with_BAQ", "without": "without_BAQ",
                     "extend": "extend_BAQ"}

    def _transcript_snp(self, fasta, out_table_prefix, type_,
                        prefix, bam_datas, table_path, args_snp):
        seq_path = os.path.join(self.seq_path, self.baqs[type_], prefix)
        for bam in bam_datas:
            stat_prefix = os.path.join(self.stat_path, "_".join([
                "stat", "_".join([prefix, self.baqs[type_], bam["sample"]]),
                "SNP"]))
            snp_file = os.path.join(self.outputs["raw"], prefix, "_".join(
                [prefix, self.baqs[type_], bam["sample"] + ".vcf"]))
            snp_detect(
                fasta, snp_file, self.outputs["depth"] + bam["sample"],
                "_".join([out_table_prefix, bam["sample"]]),
                os.path.join(seq_path, "_".join([prefix, bam["sample"]])),
                bam["bam_number"], stat_prefix, args_snp, bam["rep"])
            self.helper.move_all_content(table_path, self.fig_path, [".png"])

    def _get_para(self, args_snp):
        if args_snp.caller == "c":
            bcf_para = "-vcO"
        else:
            bcf_para = "-vmO"
        return bcf_para

    def _run_tools(self, fasta_file, type_, args_snp, bam_datas):
        bcf_para = self._get_para(args_snp)
        for bam in bam_datas:
            bam_file = os.path.join(args_snp.out_folder,
                                    bam["sample"] + ".bam")
            if type_ == "with":
                command = [args_snp.samtools_path, "mpileup", "-t", "DP"]
            elif type_ == "without":
                command = [args_snp.samtools_path, "mpileup", "-t", "DP", "-B"]
            elif type_ == "extend":
                command = [args_snp.samtools_path, "mpileup", "-t", "DP", "-E"]
            if args_snp.rg:
                command = command + ["-ugf", fasta_file, bam_file]
            else:
                command = command + ["--ignore-RG", "-ugf", fasta_file, bam_file]
            os.system(" ".join(command) + ">" + self.outputs["tmp"])
            bam["vcf"] = os.path.join(self.outputs["raw"], "_".join(
                [self.baqs[type_], bam["sample"] + ".vcf"]))
            if args_snp.chrom == "1":
                call([args_snp.bcftools_path, "call", "--ploidy", args_snp.chrom,
                      self.outputs["tmp"], bcf_para, "v", "-o", bam["vcf"]])
            elif args_snp.chrom == "2":
                call([args_snp.bcftools_path, "call",
                      self.outputs["tmp"], bcf_para, "v", "-o", bam["vcf"]])

    def _parse_vcf_by_fa(self, args_snp, type_, num_prog):
        seq_names = []
        fa_prefixs = []
        for fa in os.listdir(args_snp.fastas):
            if (fa != "all.fa") and (not fa.endswith(".fai")):
                with open(os.path.join(args_snp.fastas, fa)) as fh:
                    for line in fh:
                        line = line.strip()
                        if line.startswith(">"):
                            seq_names.append(line[1:])
                fa_prefix = ".".join(fa.split(".")[:-1])
                fa_prefixs.append(fa_prefix)
                vcf_folder = os.path.join(
                    self.outputs["raw"], fa_prefix)
                if num_prog == 0:
                    self.helper.check_make_folder(vcf_folder)
                    self.helper.check_make_folder(os.path.join(
                        self.outputs["table"], fa_prefix))
                self.helper.check_make_folder(
                    os.path.join(self.seq_path, self.baqs[type_], fa_prefix))
                for vcf in os.listdir(self.outputs["raw"]):
                    if vcf.endswith(".vcf"):
                        out = open(os.path.join(vcf_folder, "_".join(
                            [fa_prefix, vcf])), "w")
                        with open(os.path.join(self.outputs["raw"],
                                  vcf)) as vh:
                            for line in vh:
                                line = line.strip()
                                if line.startswith("#"):
                                    out.write(line + "\n")
                                else:
                                    if line.split("\t")[0] in seq_names:
                                        out.write(line + "\n")
                        out.close()
        for vcf in os.listdir(self.outputs["raw"]):
            if vcf.endswith(".vcf"):
                os.remove(os.path.join(self.outputs["raw"], vcf))
        return fa_prefixs

    def _run_sub(self, args_snp, all_fasta, type_, bam_datas, num_prog):
        self._run_tools(all_fasta, type_, args_snp, bam_datas)
        fa_prefixs = self._parse_vcf_by_fa(args_snp, type_, num_prog)
        for fa_prefix in fa_prefixs:
            for fasta in os.listdir(args_snp.fastas):
                if fa_prefix in fasta:
                    fasta_file = os.path.join(args_snp.fastas, fasta)
            table_path = os.path.join(self.outputs["table"], fa_prefix)
            table_prefix = os.path.join(table_path, "_".join(
                [fa_prefix, self.baqs[type_]]))
            self._transcript_snp(
                fasta_file, table_prefix,
                type_, fa_prefix, bam_datas, table_path, args_snp)

    def _run_program(self, all_fasta, bam_datas, args_snp):
        num_prog = 0
        for index in args_snp.program:
            if index == "with_BAQ":
                type_ = "with"
                print("Running SNP calling with BAQ")
            elif index == "without_BAQ":
                type_ = "without"
                print("Running SNP calling without BAQ")
            elif index == "extend_BAQ":
                print("Running SNP calling extend BAQ")
                type_ = "extend"
            else:
                print("Error: No correct program, please assign "
                      "\"with_BAQ\", \"without_BAQ\", \"extend_BAQ\"!")
                sys.exit()
            self._run_sub(args_snp, all_fasta, type_, bam_datas, num_prog)
            num_prog += 1

    def _run_bam(self, samtools_path, sub_command, bam_file, type_file):
        if sub_command == "merge":
            command = (" ".join([samtools_path, sub_command,
                       self.bams["whole"], bam_file]))
        elif sub_command == "sort":
            if type_file == "all":
                command = (" ".join([samtools_path, sub_command,
                                     "-o", bam_file, self.bams["whole"]]))
            else:
                command = (" ".join([samtools_path, sub_command,
                                     "-o",
                                     bam_file, type_file]))
        os.system(command)

    def _merge_bams(self, args_snp, bam_datas):
        bams = []
        num_normal = 0
        num_frag = 0
        for bam in bam_datas:
            bam["bam_number"] = 0
            out_bam = os.path.join(args_snp.out_folder, bam["sample"] + ".bam")
            if len(bam["bams"]) == 1:
                print("Sorting BAM file of " + bam["sample"])
                self._run_bam(
                    args_snp.samtools_path, "sort",
                    out_bam, bam["bams"][0])
                bam["bam_number"] = 1
            else:
                print("Merging BAM files of " + bam["sample"])
                self._run_bam(args_snp.samtools_path, "merge",
                              " ".join(bam["bams"]), "all")
                print("Sorting BAM files of " + bam["sample"])
                self._run_bam(
                    args_snp.samtools_path, "sort",
                    out_bam, "all")
                bam["bam_number"] += 1
            if os.path.exists(self.bams["whole"]):
                os.remove(self.bams["whole"])
            out_depth = open(self.outputs["depth"] + bam["sample"], "w")
            call([args_snp.samtools_path, "index",  out_bam])
            call([args_snp.samtools_path, "depth",  out_bam],
                 stdout=out_depth)
            out_depth.close()

    def _modify_header(self, fastas):
        for fasta in os.listdir(fastas):
            if fasta.endswith("fasta") or \
               fasta.endswith("fa") or \
               fasta.endswith("fna"):
                self.seq_editer.modify_header(os.path.join(fastas, fasta))

    def _get_header(self, samtools_path, bam, seq_names):
        command = " ".join([samtools_path, "view", "-H", bam])
        os.system(">".join([command, self.header]))
        fh = open(self.header, "r")
        for row in csv.reader(fh, delimiter="\t"):
            if row[0] == "@SQ":
                if row[1].split(":")[1] not in seq_names:
                    seq_names.append(row[1].split(":")[1])
        fh.close()

    def _get_genome_name(self, args_snp, bam_datas):
        seq_names = []
        for bam in bam_datas:
            bam_file = os.path.join(args_snp.out_folder,
                                    bam["sample"] + ".bam")
            self._get_header(args_snp.samtools_path,
                             bam_file, seq_names)
        return seq_names

    def _remove_bams(self, bam_datas, args_snp):
        for bam in bam_datas:
            bam_file = os.path.join(args_snp.out_folder,
                                    bam["sample"] + ".bam")
            if os.path.exists(bam_file):
                os.remove(bam_file)
            if os.path.exists(bam_file + ".bai"):
                os.remove(bam_file + ".bai")
            if os.path.exists(self.header):
                os.remove(self.header)
            os.remove(self.outputs["depth"] + bam["sample"])

    def _extract_bams(self, bams):
        bam_datas = []
        for bam in bams:
            datas = bam.split(":")
            if len(datas) != 3:
                print("Error: the format of --bam_files is wrong!")
                sys.exit()
            try:
                int(datas[1])
            except ValueError:
                print("Error: the format of --bam_files is wrong! "
                      "The format should be like "
                      "$SET_NAME:$SAMPLE_NUM:$BAMFILE1,$BAMFILE2,... "
                      "and $SAMPLE_NUM should be integer!")
                sys.exit()
            for file_ in datas[-1].split(","):
                if not os.path.exists(file_):
                    print("Error: there are some Bam files "
                          "which are not exist!")
                    sys.exit()
            bam_datas.append({"sample": datas[0], "rep": int(datas[1]),
                              "bams": datas[-1].split(",")})
        return bam_datas

    def _merge_fasta(self, fastas):
        all_fasta = os.path.join(fastas, "all.fa")
        names = []
        out = open(all_fasta, "w")
        print_ = False
        for fasta in os.listdir(fastas):
            if (fasta.endswith(".fa")) or (
                    fasta.endswith(".fasta")) or (
                    fasta.endswith(".fna")):
                with open(os.path.join(fastas, fasta)) as fh:
                    for line in fh:
                        line = line.strip()
                        if line.startswith(">"):
                            if line not in names:
                                print_ = True
                                names.append(line)
                            else:
                                print_ = False
                        if print_:
                            out.write(line + "\n")
        out.close()
        return all_fasta

    def run_snp_calling(self, args_snp):
        self._modify_header(args_snp.fastas)
        all_fasta = self._merge_fasta(args_snp.fastas)
        bam_datas = self._extract_bams(args_snp.bams)
        self._merge_bams(args_snp, bam_datas)
        if ("with_BAQ" not in args_snp.program) and (
                "without_BAQ" not in args_snp.program) and (
                "extend_BAQ" not in args_snp.program):
            print("Error: Please assign a correct programs: "
                  "\"with_BAQ\", \"without_BAQ\", \"extend_BAQ\".")
            sys.exit()
        else:
            print("Detecting mutations now")
            self._run_program(all_fasta, bam_datas, args_snp)
            os.remove(self.outputs["tmp"])
            os.remove(all_fasta)
            os.remove(all_fasta + ".fai")
        self.helper.remove_tmp_dir(args_snp.fastas)
        self._remove_bams(bam_datas, args_snp)



import os
import csv
import shutil
from copy import deepcopy
from annogesiclib.gff3 import Gff3Parser


def import_data(row, type_, args_srna, term_path):
    if type_ == "gff":
        data = {"strain": row[0], "name": row[1], "start": int(row[2]),
                "end": int(row[3]), "strand": row[4], "conds": row[5],
                "detect": row[6], "tss_pro": row[7], "end_pro": row[8],
                "avg": float(row[9]), "high": float(row[10]),
                "low": float(row[11]), "overlap_CDS": row[13],
                "overlap_percent": row[14]}
        if (term_path is not None) and (args_srna.promoter_table is None):
            data["track"] = row[12]
            data["with_term"] = row[15]
        elif (term_path is None) and (args_srna.promoter_table is not None):
            data["track"] = row[12]
            data["promoter"] = row[15]
        elif (term_path is not None) and (
                args_srna.promoter_table is not None):
            data["track"] = row[12]
            data["with_term"] = row[15]
            data["promoter"] = row[16]
        else:
            data["track"] = row[12]
        if args_srna.import_info is None:
            data["track"] = row[12]
        return data
    elif type_ == "nr":
        if len(row) == 8:
            return {"strain": row[0], "name": row[1], "strand": row[2],
                    "start": int(row[3]), "end": int(row[4]),
                    "hits": "|".join(row[5:8])}
        elif len(row) == 6:
            return {"strain": row[0], "name": row[1], "strand": row[2],
                    "start": int(row[3]), "end": int(row[4]), "hits": row[5]}
    elif type_ == "sRNA":
        if len(row) == 7:
            return {"strain": row[0], "name": row[1], "strand": row[2],
                    "start": int(row[3]), "end": int(row[4]),
                    "hits": "|".join(row[5:])}
        elif len(row) == 6:
            return {"strain": row[0], "name": row[1], "strand": row[2],
                    "start": int(row[3]), "end": int(row[4]), "hits": row[5]}


def merge_info(blasts):
    first = True
    finals = []
    if len(blasts) != 0:
        for blast in blasts:
            if first:
                repeat = 0
                first = False
                pre_blast = deepcopy(blast)
            else:
                if (pre_blast["strain"] == blast["strain"]) and (
                        pre_blast["strand"] == blast["strand"]) and (
                        pre_blast["start"] == blast["start"]) and (
                        pre_blast["end"] == blast["end"]):
                    if (repeat < 2):
                        pre_blast["hits"] = ";".join([
                            pre_blast["hits"], blast["hits"]])
                        repeat += 1
                else:
                    repeat = 0
                    finals.append(pre_blast)
                    pre_blast = blast.copy()
        finals.append(pre_blast)
    return finals


def compare_srna_table(srna_tables, srna, final, args_srna):
    '''Get the information from sRNA table which has more details'''
    for table in srna_tables:
        tsss = []
        pros = []
        cands = []
        if (table["strain"] == srna.seq_id) and (
                table["strand"] == srna.strand) and (
                table["start"] == srna.start) and (
                table["end"] == srna.end):
            final = dict(final, **table)
            start_datas = table["tss_pro"].split(";")
            end_datas = table["end_pro"].split(";")
            tsss.append(table["start"])
            pros.append(table["end"])
            for data in start_datas:
                if "TSS" in data:
                    if table["start"] != int(data.split(":")[1][:-2]):
                        tsss.append(int(data.split(":")[1][:-2]))
                elif "Cleavage" in data:
                    if table["end"] != int(data.split(":")[1][:-2]):
                        pros.append(int(data.split(":")[1][:-2]))
            for data in end_datas:
                if "Cleavage" in data:
                    if table["end"] != int(data.split(":")[1][:-2]):
                        pros.append(int(data.split(":")[1][:-2]))
            for tss in tsss:
                for pro in pros:
                    if ((pro - tss) >= args_srna.min_len) and (
                            (pro - tss) <= args_srna.max_len):
                        cands.append("-".join([str(tss), str(pro)]))
            final["candidates"] = ";".join(cands)
            if ("tex" in table["conds"]) and (
                    "frag" in table["conds"]):
                final["type"] = "TEX+/-;Fragmented"
            elif ("tex" in table["conds"]):
                final["type"] = "TEX+/-"
            elif ("frag" in table["conds"]):
                final["type"] = "Fragmented"
    return final


def compare_blast(blasts, srna, final, hit):
    for blast in blasts:
        if (srna.seq_id == blast["strain"]) and (
                srna.strand == blast["strand"]) and (
                srna.start == blast["start"]) and (
                srna.end == blast["end"]):
            final[hit] = blast["hits"]
    return final


def compare_promoter(final, args_srna):
    '''modify the score of sRNA by comparing with promoter'''
    if "promoter" in final.keys():
        if final["promoter"] != "NA":
            final["score"] = final["avg"]*args_srna.rank_promoter
        else:
            final["score"] = final["avg"]
    else:
        final["score"] = final["avg"]
    return final


def check_keys(ref_key, final_key, srna, final):
    if ref_key in srna.attributes.keys():
        final[final_key] = srna.attributes[ref_key]
    else:
        final[final_key] = "NA"


def compare(srnas, srna_tables, nr_blasts, srna_blasts, args_srna):
    '''Check sRNA candidate pass the filters or not'''
    finals = []
    for srna in srnas:
        final = {}
        check_keys("2d_energy", "energy", srna, final)
        check_keys("nr_hit", "nr_hit_num", srna, final)
        check_keys("sRNA_hit", "sRNA_hit_num", srna, final)
        check_keys("sORF", "sORF", srna, final)
        check_keys("with_term", "with_term", srna, final)
        check_keys("end_pro", "end_pro", srna, final)
        check_keys("promoter", "promoter", srna, final)
        if srna.attributes["sRNA_type"] == "intergenic":
            final["utr"] = "Intergenic"
        elif srna.attributes["sRNA_type"] == "in_CDS":
            final["utr"] = "In_CDS"
        elif srna.attributes["sRNA_type"] == "antisense":
            final["utr"] = "Antisense"
        else:
            if "," in srna.attributes["sRNA_type"]:
                final["utr"] = "5'UTR_derived;3'UTR_derived"
            elif srna.attributes["sRNA_type"] == "5utr":
                final["utr"] = "5'UTR_derived"
            elif srna.attributes["sRNA_type"] == "3utr":
                final["utr"] = "3'UTR_derived"
            elif srna.attributes["sRNA_type"] == "interCDS":
                final["utr"] = "InterCDS"
        final = compare_srna_table(srna_tables, srna, final, args_srna)
        final = compare_blast(nr_blasts, srna, final, "nr_hit")
        final = compare_blast(srna_blasts, srna, final, "sRNA_hit")
        final = compare_promoter(final, args_srna)
        finals.append(final)
    return finals


def change_srna_name(final):
    '''get the proper name of sRNA'''
    names = []
    num = 0
    for hit in final["sRNA_hit"].split(";"):
        hit_name = hit.split("|")[-2]
        hit_name = hit_name[0].upper() + hit_name[1:]
        num += 1
        if "Sau" in hit_name:
            sau = hit_name.split("-")
            if len(sau) == 1:
                hit_name = hit_name[:3] + "-" + hit_name[3:]
        if hit_name not in names:
            names.append(hit_name)
        if num == 3:
            break
    return names


def print_file(finals, out, srnas, out_gff):
    rank = 1
    for final in finals:
        names = [final["name"]]
        if "nr_hit" not in final.keys():
            final["nr_hit"] = "NA"
        if "sRNA_hit" not in final.keys():
            final["sRNA_hit"] = "NA"
        if "with_term" not in final.keys():
            final["with_term"] = "NA"
        if "promoter" not in final.keys():
            final["promoter"] = "NA"
        if final["sRNA_hit"] != "NA":
            names = change_srna_name(final)
        length = final["end"] - final["start"]
        out.write("\t".join([str(rank),
                  final["strain"], "/".join(names), str(final["start"]),
                  str(final["end"]), final["strand"], final["tss_pro"],
                  final["end_pro"], final["candidates"], final["type"],
                  str(final["avg"]), str(final["high"]), str(final["low"]),
                  final["track"], final["energy"], final["utr"], final["sORF"],
                  final["nr_hit_num"], final["sRNA_hit_num"],
                  final["nr_hit"], final["sRNA_hit"], final["overlap_CDS"],
                  str(final["overlap_percent"]), final["with_term"],
                  final["promoter"], str(length)]) + "\n")
        rank += 1
    for srna in srnas:
        for final in finals:
            if (srna.seq_id == final["strain"]) and (
                    srna.start == final["start"]) and (
                    srna.end == final["end"]) and (
                    srna.strand == final["strand"]):
                if ("sRNA_hit" in final.keys()):
                    if final["sRNA_hit"] != "NA":
                        names = change_srna_name(final)
                        srna.attributes["Name"] = "/".join(names)
        attribute_string = ";".join(
            ["=".join(items) for items in srna.attributes.items()])
        out_gff.write("\t".join([srna.info_without_attributes,
                                 attribute_string]) + "\n")


def read_table(srna_table_file, nr_blast, srna_blast_file, args_srna,
               term_path):
    srna_tables = []
    nr_blasts = []
    srna_blasts = []
    f_h = open(srna_table_file, "r")
    for row in csv.reader(f_h, delimiter='\t'):
        srna_tables.append(import_data(row, "gff", args_srna, term_path))
    f_h.close()
    if os.path.exists(nr_blast):
        f_h = open(nr_blast, "r")
        for row in csv.reader(f_h, delimiter='\t'):
            nr_blasts.append(import_data(row, "nr", args_srna, term_path))
        f_h.close()
    if os.path.exists(srna_blast_file):
        f_h = open(srna_blast_file, "r")
        for row in csv.reader(f_h, delimiter='\t'):
            srna_blasts.append(import_data(row, "sRNA", args_srna, term_path))
        f_h.close()
    return srna_tables, nr_blasts, srna_blasts


def read_gff(srna_gff):
    srnas = []
    for entry in Gff3Parser().entries(open(srna_gff)):
        srnas.append(entry)
    srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    return srnas


def gen_srna_table(srna_gff, srna_table_file, nr_blast, srna_blast_file,
                   args_srna, out_file, term_path):
    '''generate the sRNA table for more details'''
    srnas = read_gff(srna_gff)
    srna_tables, nr_blasts, srna_blasts = read_table(
        srna_table_file, nr_blast, srna_blast_file, args_srna, term_path)
    out = open(out_file, "w")
    tmp_gff = out_file.replace(".csv", ".gff")
    out_gff = open(tmp_gff, "w")
    out_gff.write("##gff-version 3\n")
    out.write("\t".join([
        "rank", "strain", "name", "start", "end", "strand",
        "start_with_TSS/Cleavage_site", "end_with_cleavage", "candidates",
        "lib_type", "best_avg_coverage", "best_highest_coverage",
        "best_lowest_coverage", "track/coverage",
        "normalized_secondary_energy_change(by_length)", "sRNA_types",
        "conflict_sORF", "nr_hit_number", "sRNA_hit_number",
        "nr_hit_top3|ID|e-value", "sRNA_hit|e-value", "overlap_CDS",
        "overlap_percent", "end_with_terminator",
        "associated_promoter", "sRNA_length"]) + "\n")
    nr_blasts = merge_info(nr_blasts)
    srna_blasts = merge_info(srna_blasts)
    finals = compare(srnas, srna_tables, nr_blasts,
                     srna_blasts, args_srna)
    sort_finals = sorted(finals, key=lambda x: (x["score"]), reverse=True)
    print_file(sort_finals, out, srnas, out_gff)
    out_gff.close()
    shutil.move(tmp_gff, srna_gff)


def print_best(detect, out, srna):
    no_print = False
    for key, value in detect.items():
        if not value:
            no_print = True
    if not no_print:
        out.write(srna.info + "\n")


def check_energy(srna, energy, detect):
    '''check the folding energy of sRNA'''
    if "2d_energy" in srna.attributes.keys():
        if float(srna.attributes["2d_energy"]) < energy:
            detect["energy"] = True
    else:
        detect["energy"] = True


def check_tss(import_info, srna, detect):
    '''check the sRNA is associated with TSS or not'''
    if import_info is not None:
        if "tss" in import_info:
            if "with_TSS" in srna.attributes.keys():
                if srna.attributes["with_TSS"] != "NA":
                    detect["TSS"] = True
                elif (srna.attributes["sRNA_type"] != "intergenic") and (
                      srna.attributes["sRNA_type"] != "in_CDS") and (
                      srna.attributes["sRNA_type"] != "antisense"):
                    if (("3utr" in srna.attributes["sRNA_type"]) or (
                         "interCDS" in srna.attributes["sRNA_type"])) and (
                         srna.attributes["start_cleavage"] != "NA"):
                        detect["TSS"] = True
        else:
            detect["TSS"] = True
    else:
        detect["TSS"] = True


def check_nr_hit(srna, nr_hits_num, detect, import_info):
    '''check the sRNA has hit in nr database or not'''
    if import_info is not None:
        if ("nr_hit" in srna.attributes.keys()) and (
                "blast_nr" in import_info):
            if (srna.attributes["nr_hit"] == "NA") or (
                    int(srna.attributes["nr_hit"]) <= nr_hits_num):
                detect["nr_hit"] = True
        else:
            detect["nr_hit"] = True
    else:
        detect["nr_hit"] = True


def check_sorf(import_info, srna, detect):
    '''check the sRNA is overlap with sORF or not'''
    if import_info is not None:
        if ("sorf" in import_info):
            if ("sORF" in srna.attributes.keys()):
                if srna.attributes["sORF"] == "NA":
                    detect["sORF"] = True
        else:
            detect["sORF"] = True
    else:
        detect["sORF"] = True


def check_srna_hit(srna, import_info, detect):
    '''check the sRNA has hit in sRNA database or not'''
    if import_info is not None:
        if ("sRNA_hit" in srna.attributes.keys()) and (
                "blast_srna" in import_info):
            if (srna.attributes["sRNA_hit"] != "NA"):
                for key in detect.keys():
                    detect[key] = True
            else:
                count = 0
                for value in detect.values():
                    if value:
                        count += 1
                if count == 4:
                    detect["sRNA_hit"] = True
        else:
            detect["sRNA_hit"] = True
    else:
        detect["sRNA_hit"] = True


def check_term(import_info, srna, detect):
    '''check the sRNA is associated with terminator or not'''
    if import_info is not None:
        if "term" in import_info:
            if ("with_term" in srna.attributes.keys()):
                if srna.attributes["with_term"] != "NA":
                    detect["term"] = True
                elif ("end_cleavage" in srna.attributes.keys()):
                    if srna.attributes["end_cleavage"] != "NA":
                        detect["term"] = True
        else:
            detect["term"] = True
    else:
        detect["term"] = True


def check_promoter(import_info, srna, detect):
    '''check the sRNA is associated with promoter or not'''
    if import_info is not None:
        if "promoter" in import_info:
            if ("promoter" in srna.attributes.keys()):
                if srna.attributes["promoter"] != "NA":
                    detect["promoter"] = True
        else:
            detect["promoter"] = True
    else:
        detect["promoter"] = True


def gen_best_srna(srna_file, out_file, args_srna):
    '''generate the best sRNA candidates'''
    srnas = read_gff(srna_file)
    out = open(out_file, "w")
    out.write("##gff-version 3\n")
    for srna in srnas:
        detect = {"energy": False, "TSS": False, "nr_hit": False,
                  "sRNA_hit": False, "sORF": False, "term": False,
                  "promoter": False}
        check_energy(srna, args_srna.energy, detect)
        check_tss(args_srna.import_info, srna, detect)
        check_nr_hit(srna, args_srna.nr_hits_num, detect,
                     args_srna.import_info)
        check_sorf(args_srna.import_info, srna, detect)
        check_srna_hit(srna, args_srna.import_info, detect)
        check_term(args_srna.import_info, srna, detect)
        check_promoter(args_srna.import_info, srna, detect)
        print_best(detect, out, srna)
    out.close()



import csv


class SpliceParser(object):
    '''parser the splice data of segemehl'''

    def parser(self, splice_fh):
        for row in csv.reader(splice_fh, delimiter="\t"):
            yield assign_value(row)


class assign_value(object):

    def __init__(self, row):
        self.strain = row[0]
        self.start = int(row[1])
        self.end = int(row[2])
        self.splice = row[3]
        splice = row[3].split(":")
        self.supported_reads = int(splice[1])
        self.start_site_reads = int(splice[2])
        self.end_site_reads = int(splice[3])
        self.splice_type = splice[4]
        self.situation = splice[5]
        self.strand = row[5]
        self.info = ("\t".join(row))

    def __str__(self):
        return "{0} {1} {2} {3} {4}".format(
               self.strain, self.start, self.end, self.splice, self.strand)



import os
import csv
import shutil
from annogesiclib.gff3 import Gff3Parser


def read_file(gff_file, args_srna):
    srnas = []
    for entry in Gff3Parser().entries(open(gff_file)):
        attributes = {}
        for key, value in entry.attributes.items():
            if "promoter" not in key:
                attributes[key] = value
        entry.attributes = attributes
        srnas.append(entry)
    srnas = sorted(srnas, key=lambda k: (k.seq_id, k.start, k.end, k.strand))
    fh = open(args_srna.promoter_table, "r")
    pros = []
    for row in csv.reader(fh, delimiter='\t'):
        if (row[0] != "strain") and (
                row[3] in args_srna.promoter_name):
            pros.append({"strain": row[0], "pos": row[1],
                         "strand": row[2], "name": row[3]})
    fh.close()
    return srnas, pros


def print_table(srna_table, out_t, srnas):
    fh = open(srna_table, "r")
    for row in csv.reader(fh, delimiter='\t'):
        for srna in srnas:
            if (row[0] == srna.seq_id) and (
                    int(row[2]) == srna.start) and (
                    int(row[3]) == srna.end) and (
                    row[4] == srna.strand):
                if "promoter" in srna.attributes.keys():
                    promoter = [srna.attributes["promoter"]]
                else:
                    promoter = ["NA"]
                out_t.write("\t".join(row + promoter) + "\n")


def compare_srna_promoter(srna_gff, srna_table, args_srna):
    '''compare sRNA and promoter to find the sRNA 
    which is associated with a promoter.
    it is for the ranking of sRNA'''
    srnas, pros = read_file(srna_gff, args_srna)
    out_g = open("tmp_srna.gff", "w")
    out_t = open("tmp_srna.csv", "w")
    out_g.write("##gff-version 3\n")
    for srna in srnas:
        tsss = []
        detect = False
        if "with_TSS" in srna.attributes.keys():
            if srna.attributes["with_TSS"] != "NA":
                datas = srna.attributes["with_TSS"].split(",")
                for data in datas:
                    info = data.split(":")[-1]
                    tss = info.split("_")
                    tsss.append({"pos": tss[0], "strand": tss[-1]})
        if len(tsss) != 0:
            for tss in tsss:
                for pro in pros:
                    if (srna.seq_id == pro["strain"]) and (
                            tss["strand"] == pro["strand"]) and (
                            tss["pos"] == pro["pos"]):
                        detect = True
                        if "promoter" not in srna.attributes.keys():
                            srna.attributes["promoter"] = pro["name"]
                        else:
                            srna.attributes["promoter"] = ",".join([
                                srna.attributes["promoter"],
                                pro["name"]])
        if detect:
            out_g.write(srna.info + ";promoter=" +
                        srna.attributes["promoter"] + "\n")
        else:
            out_g.write(srna.info + ";promoter=NA" + "\n")
    print_table(srna_table, out_t, srnas)
    os.remove(srna_gff)
    os.remove(srna_table)
    out_t.close()
    out_g.close()
    shutil.move("tmp_srna.gff", srna_gff)
    shutil.move("tmp_srna.csv", srna_table)



import os
import shutil

def mod_file(input_file, out, indexs):
    with open(input_file) as fh:
        for line in fh:
            line = line.strip()
            if line.startswith(">"):
                out.write(indexs[line] + "\n")
            else:
                out.write(line + "\n")
    out.close()

def extract_info_sec(sec_file, seq_file, index_file):
    out_sec = open(sec_file + "tmp", "w")
    out_seq = open(seq_file + "tmp", "w")
    indexs = {}
    with open(index_file) as hi:
        for line in hi:
            line = line.strip()
            if line.startswith(">"):
                tag = line.split("|")[0]
                indexs[tag] = line
    mod_file(sec_file, out_sec, indexs)
    mod_file(seq_file, out_seq, indexs)
    os.remove(sec_file)
    shutil.move(sec_file + "tmp", sec_file)
    os.remove(seq_file)
    shutil.move(seq_file + "tmp", seq_file)

def modify_header(seq_file, index_file):
    out = open(seq_file, "w")
    with open(index_file) as fh:
        for line in fh:
            line = line.strip()
            if line.startswith(">"):
                tag = line.split("|")[0]
                out.write(tag + "\n")
            else:
                out.write(line + "\n")
